#include "dxstdafx.h"
#include "Commander.h"
#include "Leader.h"
#include "database.h"
#include "movement.h"
#include "body.h"
#include "terrain.h"
#include <algorithm>

#define PLYDEAD 4

//Add new states here
enum StateName {
	STATE_Initialize,	//Note: the first enum is the starting state
	STATE_Patrol,
	STATE_Idle,
	STATE_Attacking,
	STATE_Radioing,
	STATE_Investigate,
	STATE_Engage,
	STATE_Dead
};

//Add new substates here
enum SubstateName {
	//empty
};

bool Commander::States( State_Machine_Event event, MSG_Object * msg, int state, int substate )
{
BeginStateMachine

	//Global message responses
	OnMsg( MSG_Reset )
		ResetStateMachine();

	OnMsg( MSG_SetHeuristic )
		m_heuristicWeight = msg->GetFloatData();

	OnMsg( MSG_KillRadio )
		m_radio = false;

	OnMsg(MSG_Die)
		ChangeState(STATE_Dead);

	//The Commander needs to be able to act whenever he receives a response
	OnMsg(MSG_StatusRequest)
		if(m_radio && m_squadList.size() > 0) 
		{
			if(m_squadList[m_nextLeaderIndex]->IsAlive())
			{
				m_owner->GetTiny().PlaySoundEffect(SOUND_STATUS_REQUEST);
				SendMsgDelayed(2.0f, MSG_StatusRequest, m_squadList[m_nextLeaderIndex]->GetID());
			}
			else
			{
				m_nextLeaderIndex++;
				if(m_nextLeaderIndex >= m_squadList.size()) m_nextLeaderIndex = 0;
			}
		}
		else StopTimer(MSG_StatusRequest);

	OnMsg(MSG_OK)
		if(m_radio && m_squadList.size() > 0)
		{
			//play sound
			m_nextLeaderIndex++;
			if(m_nextLeaderIndex >= m_squadList.size()) m_nextLeaderIndex = 0;
		}

	OnMsg(MSG_NoResponse)
		if(msg->IsDataValid())
		{
			m_lastKnownSighting = m_squadList[msg->GetIntData()]->GetBody().GetPos();
			RadioInformation* info = new RadioInformation(m_lastKnownSighting, msg->GetIntData());
			objectID id = FindNearestSquadToAssist(m_lastKnownSighting, msg->GetIntData());
			if(id != m_owner->GetID() && m_radio) 
			{	
				m_owner->GetTiny().PlaySoundEffect(SOUND_SEND_INVESTIGATE);
				SendMsgDelayed(2.0f, MSG_Investigate, id, (int)info);
			}
			else 
			{
				delete info;
				m_owner->GetTiny().PlaySoundEffect(SOUND_SELF_INVESTIGATE);
				ChangeState(STATE_Investigate);
			}
		}
		else assert(!1);	//Should never happen!
		m_nextLeaderIndex++;
		if(m_nextLeaderIndex >= m_squadList.size()) m_nextLeaderIndex = 0;

	OnMsg(MSG_EnemySighted)
		if(msg->IsDataValid() && m_radio && g_terrain.m_player->GetStateMachine()->GetState() != PLYDEAD)
		{
			m_lastKnownSighting = ((RadioInformation*)(msg->GetIntData()))->location;
			unsigned engagedSquadIndex = ((RadioInformation*)(msg->GetIntData()))->squadIndex;
			objectID id = FindNearestSquadToAssist(m_lastKnownSighting, engagedSquadIndex);
			if(id != m_owner->GetID()) 
			{
				m_owner->GetTiny().PlaySoundEffect(SOUND_SEND_ENGAGE);
				SendMsgDelayed(2.0f, MSG_EnemySighted, id, msg->GetIntData());	//The Receiver Must Delete the Data!
			}
			else 
			{
				delete (RadioInformation*)(msg->GetIntData());
				m_owner->GetTiny().PlaySoundEffect(SOUND_SELF_ENGAGE);
				ChangeState(STATE_Engage);
			}
		}

	OnMsg(MSG_CallForReinforcements)
		if(msg->IsDataValid() && m_radio)
		{
			//The Squad sending this message has been terminated.
			//Send in a new squad to take their place.
			m_owner->GetTiny().PlaySoundEffect(SOUND_CALL_FOR_REINFORCEMENTS);
			GenerateNewSquad(msg->GetIntData());
		}

	OnMsg(MSG_Investigate)
		if(msg->IsDataValid() && m_radio)
		{
			m_lastKnownSighting = ((RadioInformation*)(msg->GetIntData()))->location;
			unsigned engagedSquadIndex = ((RadioInformation*)(msg->GetIntData()))->squadIndex;
			objectID id = FindNearestSquadToAssist(m_lastKnownSighting, engagedSquadIndex);
			if(id != m_owner->GetID()) 
			{
				m_owner->GetTiny().PlaySoundEffect(SOUND_SEND_INVESTIGATE);
				SendMsg(MSG_Investigate, id, msg->GetIntData());	//The Receiver Must Delete the Data!
			}
			else 
			{
				delete (RadioInformation*)(msg->GetIntData());
				m_owner->GetTiny().PlaySoundEffect(SOUND_SELF_INVESTIGATE);
				ChangeState(STATE_Investigate);
			}
		}

	OnMsg(MSG_LeaderDead)
		if(msg->IsDataValid() && m_radio) 
		{
			//m_owner->GetTiny().PlaySoundEffect(SOUND_SEND_INVESTIGATE);
			SelectNewLeader(m_squadList[msg->GetIntData()]);
			//delete (RadioInformation*)(msg->GetIntData());
			//SelectNewLeader(m_squadList[squadIndex]);
		}

	OnMsg(MSG_EnemyDead)
		curRadioMessage = RETURN_TO_SURVEILLANCE;
		ChangeState(STATE_Radioing);

	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Initialize )

		OnEnter
			m_owner->GetMovement().SetIdleSpeed();
			SetTimer(10.0f, MSG_StatusRequest, NO_SCOPING);
			m_nextLeaderIndex = 0;
			m_heuristicWeight = 1.0f;
			ChangeState(STATE_Idle);

	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Patrol )

		OnEnter
			m_owner->GetMovement().SetWalkSpeed();
			int r = 0, c = 0;
			if(m_patrolList.size() > 0)
			{
				g_terrain.GetRowColumn(&(m_patrolList[index]), &r, &c);
				m_owner->GetMovement().ComputePath( r, c, m_heuristicWeight);
			}
			else ChangeState(STATE_Idle);

		OnUpdate
			//do line of sight check
			if(PlayerVisible() == 2)
			{
				m_lastKnownSighting = g_terrain.m_player->GetBody().GetPos();
				curRadioMessage = ENEMY_SIGHTED;
				ChangeState(STATE_Radioing);
				//Add a check for if there is a need for New Squads to be Generated
			}
			else if(PlayerVisible() == 1)
			{
				m_lastKnownSighting = g_terrain.m_player->GetBody().GetPos();
				m_owner->GetTiny().PlaySoundEffect(SOUND_INVESTIGATE);
				ChangeState(STATE_Investigate);
			}

		OnMsg(MSG_Arrived)
			index = (index >= m_patrolList.size() - 1)? 0: index+1;
			ChangeState(STATE_Idle);

		//OnMsg(MSG_StatusRequest)
		//OnMsg(MSG_OK)
		//OnMsg(MSG_NoResponse)
		//OnMsg(MSG_Investigate)
		//OnMsg(MSG_ReplaceSquad)
		//OnMsg(MSG_CallForReinforcements)
		//OnMsg(MSG_EnemySighted)
		//OnMsg(MSG_EnemyDead)
		//OnMsg(MSG_ReturnToSurveillance)
		//OnMsg(MSG_LeaderDead)

	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Idle )

		OnEnter
			m_owner->GetMovement().SetIdleSpeed();
			ChangeStateDelayed(5.0f, STATE_Patrol);

		OnUpdate
			if(PlayerVisible() == 2)
			{
				m_lastKnownSighting = g_terrain.m_player->GetBody().GetPos();
				curRadioMessage = ENEMY_SIGHTED;
				ChangeState(STATE_Radioing);
			}
			else if(PlayerVisible() == 1)
			{
				m_lastKnownSighting = g_terrain.m_player->GetBody().GetPos();
				m_owner->GetTiny().PlaySoundEffect(SOUND_INVESTIGATE);
				ChangeState(STATE_Investigate);
			}

		//OnMsg(MSG_StatusRequest)
		//OnMsg(MSG_OK)
		//OnMsg(MSG_NoResponse)
		//OnMsg(MSG_Investigate)
		//OnMsg(MSG_ReplaceSquad)
		//OnMsg(MSG_CallForReinforcements)
		//OnMsg(MSG_EnemySighted)
		//OnMsg(MSG_EnemyDead)
		//OnMsg(MSG_ReturnToSurveillance)
		//OnMsg(MSG_LeaderDead)

	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Attacking )
		/*
			It is presumed that m_lastKnownSighting has been filled with a current location
			of the enemy, as the commander will only attack when the enemy is sighted
			set goal to a location near the enemy, with a LOS
		*/
		OnEnter
			int r = 0, c = 0;
			g_terrain.GetRowColumn(&(m_lastKnownSighting), &r, &c);
			m_owner->GetMovement().ComputePath( r, c, m_heuristicWeight);
			m_owner->GetMovement().SetJogSpeed();
		OnUpdate
			dbCompositionList list;
			g_database.ComposeList(list, OBJECT_Player);
			int r, c, ar, ac;
			g_terrain.GetRowColumn(&(g_terrain.m_player->GetBody().GetPos()), &r, &c);
			g_terrain.GetRowColumn(&(m_owner->GetBody().GetPos()), &ar, &ac);
			//if the player has moved check if we can still see him
			if((m_owner->GetMovement().goalr != r || 
				m_owner->GetMovement().goalc != c) && list[0]->GetStateMachine()->GetState() != PLYDEAD)
			{
				//if we can still see him, then recompute the path to his new location
				D3DXVECTOR3 direction = g_terrain.m_player->GetBody().GetPos() - m_owner->GetBody().GetPos();
				if(g_terrain.LOSCheck(&m_owner->GetBody().GetPos(), &direction) == NULL)
				{
					m_lastKnownSighting = g_terrain.m_player->GetBody().GetPos();
					m_owner->GetMovement().ComputePath( r, c, m_heuristicWeight);
				}
			}
			if(ar == r && ac == c)
			{
				SendMsg(MSG_Die, g_terrain.m_player->GetID());
				curRadioMessage = ENEMY_DEAD;
				ChangeState(STATE_Radioing);
			}

		OnMsg(MSG_StatusRequest)
		OnMsg(MSG_OK)
		OnMsg(MSG_NoResponse)
		OnMsg(MSG_Investigate)
		OnMsg(MSG_ReplaceSquad)
		OnMsg(MSG_CallForReinforcements)
		OnMsg(MSG_EnemySighted)
		//OnMsg(MSG_EnemyDead)
		//OnMsg(MSG_ReturnToSurveillance)
		OnMsg(MSG_LeaderDead)

		OnExit
			m_owner->GetMovement().SetWalkSpeed();

	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Radioing )
		OnEnter
			//go through list of squad leaders and radio each
			//after a message, the commander MUST change state
			//NOTE: We need to go over each case
			switch(curRadioMessage)
			{
				case NO_MESSAGE:
					assert(!1);
					break;

				case STATUS_REQUEST:
					if(m_radio) for(std::vector<GameObject*>::iterator iter = m_squadList.begin(); iter != m_squadList.end(); ++iter)
						SendMsgDelayed(2.0f, MSG_StatusRequest, (*iter)->GetID());
					break;

				case OK:
					if(m_radio)
					{
						m_owner->GetTiny().PlaySoundEffect(SOUND_RETURN_TO_SURVEILLANCE);
						for(std::vector<GameObject*>::iterator iter = m_squadList.begin(); iter != m_squadList.end(); ++iter) 
							SendMsgDelayed(2.0f, MSG_ReturnToSurveillance, (*iter)->GetID());
					}

					ChangeState(STATE_Patrol);
					break;

				case NO_RESPONSE:
					if(m_radio) for(std::vector<GameObject*>::iterator iter = m_squadList.begin(); iter != m_squadList.end(); ++iter)
						SendMsgDelayed(2.0f, MSG_NoResponse, (*iter)->GetID());
					break;

				case INVESTIGATE:
					if(m_radio) for(std::vector<GameObject*>::iterator iter = m_squadList.begin(); iter != m_squadList.end(); ++iter)
					{
						RadioInformation* info = new RadioInformation(m_lastKnownSighting, 0);
						SendMsgDelayed(2.0f, MSG_Investigate, (*iter)->GetID(), (int)info);
					}
					PopState();
					break;

				case REPLACE_SQUAD:
					if(m_radio) for(std::vector<GameObject*>::iterator iter = m_squadList.begin(); iter != m_squadList.end(); ++iter)
						SendMsgDelayed(2.0f, MSG_ReplaceSquad, (*iter)->GetID());
					break;

				case CALL_FOR_REINFORCEMENTS:
					if(m_radio) for(std::vector<GameObject*>::iterator iter = m_squadList.begin(); iter != m_squadList.end(); ++iter)
						SendMsgDelayed(2.0f, MSG_CallForReinforcements, (*iter)->GetID());
					break;

				case ENEMY_SIGHTED:
					if(m_radio) 
					{
						m_owner->GetTiny().PlaySoundEffect(SOUND_ENEMY_SIGHTED);
						for(std::vector<GameObject*>::iterator iter = m_squadList.begin(); iter != m_squadList.end(); ++iter)
						{
							RadioInformation* info = new RadioInformation(m_lastKnownSighting, 0);
							SendMsgDelayed(2.0f, MSG_EnemySighted, (*iter)->GetID(), (int)info);
						}
					}
					ChangeState(STATE_Attacking);
					break;

				case ENEMY_DEAD:
					if(m_radio) 
					{
						m_owner->GetTiny().PlaySoundEffect(SOUND_ENEMY_DEAD);
						for(std::vector<GameObject*>::iterator iter = m_squadList.begin(); iter != m_squadList.end(); ++iter)
							SendMsg(MSG_EnemyDead, (*iter)->GetID());
					}
					ChangeState(STATE_Patrol);
					break;

				case RETURN_TO_SURVEILLANCE:
					if(m_radio) 
					{
						m_owner->GetTiny().PlaySoundEffect(SOUND_RETURN_TO_SURVEILLANCE);
						for(std::vector<GameObject*>::iterator iter = m_squadList.begin(); iter != m_squadList.end(); ++iter)
							SendMsgDelayed(2.0f, MSG_ReturnToSurveillance, (*iter)->GetID());
					}
					ChangeState(STATE_Patrol);
					break;

				case LEADER_DEAD:
					if(m_radio) for(std::vector<GameObject*>::iterator iter = m_squadList.begin(); iter != m_squadList.end(); ++iter)
						SendMsgDelayed(2.0f, MSG_LeaderDead, (*iter)->GetID());
					break;

				default:
					assert(!1);
					break;
			}
			//PopState();		//We should never it this!

	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Investigate )
	/*
		Commander has seen or heard something, but isn't sure that it's the enemy
		The spot to investigate will be in m_lastKnownSighting
	*/
		OnEnter
			m_owner->GetMovement().SetWalkSpeed();
			int r, c;
			g_terrain.GetRowColumn(&m_lastKnownSighting, &r, &c);
			m_owner->GetMovement().ComputePath(r, c, 1.0f);

		OnUpdate
			//do los check
			if(PlayerVisible() == 2)
			{
				m_lastKnownSighting = g_terrain.m_player->GetBody().GetPos();
				curRadioMessage = ENEMY_SIGHTED;
				ChangeState(STATE_Radioing);
			}

		OnMsg(MSG_Arrived)
			if(m_radio)
			{
				curRadioMessage = OK;
				ChangeState(STATE_Radioing);
			}

			OnMsg(MSG_StatusRequest)
			OnMsg(MSG_OK)
			OnMsg(MSG_NoResponse)
			OnMsg(MSG_Investigate)
			OnMsg(MSG_ReplaceSquad)
			OnMsg(MSG_CallForReinforcements)
			//OnMsg(MSG_EnemySighted)
			//OnMsg(MSG_EnemyDead)
			//OnMsg(MSG_ReturnToSurveillance)
			//OnMsg(MSG_LeaderDead)

		//Must Write A Check For If The Commander
		//Is Investigating A Non-Responsive Group

	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Engage )
	/*
		Commander has seen or heard something, but isn't sure that it's the enemy
		The spot to investigate will be in m_lastKnownSighting
	*/
		OnEnter
			m_owner->GetMovement().SetJogSpeed();
			int r, c;
			g_terrain.GetRowColumn(&m_lastKnownSighting, &r, &c);
			m_owner->GetMovement().ComputePath(r, c, 1.0f);

		OnUpdate
			//do los check
			if(PlayerVisible() == 2)
			{
				m_lastKnownSighting = g_terrain.m_player->GetBody().GetPos();
				ChangeState(STATE_Attacking);
			}

		OnMsg(MSG_Arrived)
			curRadioMessage = OK;
			ChangeState(STATE_Radioing);

			OnMsg(MSG_StatusRequest)
			OnMsg(MSG_OK)
			OnMsg(MSG_NoResponse)
			OnMsg(MSG_Investigate)
			OnMsg(MSG_ReplaceSquad)
			OnMsg(MSG_CallForReinforcements)
			OnMsg(MSG_EnemySighted)
			//OnMsg(MSG_EnemyDead)
			//OnMsg(MSG_ReturnToSurveillance)
			OnMsg(MSG_LeaderDead)

	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Dead )
	/*
		We died.
	*/
		OnEnter
			m_owner->SetAlive(false);
			StopTimer(MSG_StatusRequest);
		OnUpdate
			if(m_owner->GetBody().GetPos().y > -0.5)
				m_owner->GetBody().GetPos().y-= 0.01;

		OnMsg(MSG_StatusRequest)
			m_owner->GetTiny().PlaySoundEffect(SOUND_NO_RESPONSE);
			SendMsgDelayed(2.0f, MSG_NoResponse, msg->GetSender(), 0);
		OnMsg(MSG_OK)
			m_owner->GetTiny().PlaySoundEffect(SOUND_NO_RESPONSE);
			SendMsgDelayed(2.0f, MSG_NoResponse, msg->GetSender(), 0);
		OnMsg(MSG_NoResponse)
			m_owner->GetTiny().PlaySoundEffect(SOUND_NO_RESPONSE);
			SendMsgDelayed(2.0f, MSG_NoResponse, msg->GetSender(), 0);
		OnMsg(MSG_Investigate)
			m_owner->GetTiny().PlaySoundEffect(SOUND_NO_RESPONSE);
			SendMsgDelayed(2.0f, MSG_NoResponse, msg->GetSender(), 0);
		OnMsg(MSG_ReplaceSquad)
			m_owner->GetTiny().PlaySoundEffect(SOUND_NO_RESPONSE);
			SendMsgDelayed(2.0f, MSG_NoResponse, msg->GetSender(), 0);
		OnMsg(MSG_CallForReinforcements)
			m_owner->GetTiny().PlaySoundEffect(SOUND_NO_RESPONSE);
			SendMsgDelayed(2.0f, MSG_NoResponse, msg->GetSender(), 0);
		OnMsg(MSG_EnemySighted)
			m_owner->GetTiny().PlaySoundEffect(SOUND_NO_RESPONSE);
			SendMsgDelayed(2.0f, MSG_NoResponse, msg->GetSender(), 0);
		OnMsg(MSG_EnemyDead)
			m_owner->GetTiny().PlaySoundEffect(SOUND_NO_RESPONSE);
			SendMsgDelayed(2.0f, MSG_NoResponse, msg->GetSender(), 0);
		OnMsg(MSG_ReturnToSurveillance)
			m_owner->GetTiny().PlaySoundEffect(SOUND_NO_RESPONSE);
			SendMsgDelayed(2.0f, MSG_NoResponse, msg->GetSender(), 0);
		OnMsg(MSG_LeaderDead)
			m_owner->GetTiny().PlaySoundEffect(SOUND_NO_RESPONSE);
			SendMsgDelayed(2.0f, MSG_NoResponse, msg->GetSender(), 0);

EndStateMachine
}

objectID Commander::FindNearestSquadToAssist(D3DXVECTOR3 areaToAssist, unsigned squadToAssist)
{
	float closestDistance = 1000000.0f;
	GameObject* closestSquad = 0;

	for(int i = 0; i < m_squadList.size(); ++i)
	{
		if(i != squadToAssist)
		{
			D3DXVECTOR3 squadPos = m_squadList[i]->GetBody().GetPos();
			D3DXVECTOR3 diff = squadPos - areaToAssist;
			float distance = D3DXVec3Length( &diff );

			if( closestSquad )
			{
				if( distance < closestDistance && closestSquad->IsAlive())
				{
					closestDistance = distance;
					closestSquad = m_squadList[i];
				}
			}
			else
			{
				closestDistance = distance;
				closestSquad = m_squadList[i];
			}
		}
	}

	if(closestSquad == 0) return m_owner->GetID();
	else return(closestSquad->GetID());
}

//Returns the ID of the New Squad's Leader
objectID Commander::GenerateNewSquad(unsigned oldSquadIndex)
{
	//If we have replacements standing by, add a squad leader
	//to the list, and deregister the old Squad Leader
	if(!m_replacementList.empty())
	{
		GameObject* newSquad = m_replacementList.back();
		m_replacementList.pop_back();
		Leader* oldLeader = (Leader*)m_squadList[oldSquadIndex];
		Leader* newLeader = (Leader*)newSquad;

		newLeader->RegisterIndex(oldSquadIndex);
		newLeader->RegisterSquadCommander(m_owner);
		newLeader->RegisterPatrol(oldLeader->GetPatrol());

		m_squadList[oldSquadIndex] = newSquad;
		return newLeader->GetID();
	}

	//If No New Squads can be Created, then give Commander's ID
	//So he will be sent to investigate, then remove the old squad
	else
	{
		std::vector<GameObject*>::iterator it = m_squadList.begin();
		while(oldSquadIndex--) ++it;
		m_squadList.erase(std::remove(m_squadList.begin(), m_squadList.end(), *it), m_squadList.end());
		(m_nextLeaderIndex == 0)? 0: m_nextLeaderIndex - 1;	//Don't want a -1 Index!
		return m_owner->GetID();
	}
}

//Choose a Private to take the reigns of his Squad
//A Boy becomes a Man
void Commander::SelectNewLeader(GameObject* oldSquadLeader)
{
	Leader* oldLeader = (Leader*)(oldSquadLeader->GetStateMachine());
	if(oldLeader->GetSquadSize() == 0) return;

	GameObject* newSquadLeader = oldLeader->GetSquad()->back();
	oldLeader->UnregisterSquadMember();
	newSquadLeader->PushStateMachine(*new Leader( *newSquadLeader ), true);
	Leader* newLeader = (Leader*)(newSquadLeader->GetStateMachine());
	for(int i = 0; i < m_squadList.size(); ++i)
		if(m_squadList[i] == oldSquadLeader)
		{
			m_squadList[i] = newSquadLeader;
			break;
		}
	newLeader->RegisterSquadCommander(m_owner);
	newLeader->RegisterIndex(oldLeader->GetIndex());
	newLeader->RegisterPatrol(oldLeader->GetPatrol());
	newLeader->RegisterSquad(oldLeader->GetSquad());
	newSquadLeader->SetName("Leader");
}

//If the player is nearby, we are facing the player (180 degree FOV) and we can
//see him directly, then visible
//2 means direct sighting, 1 means he is somewhat detected, 0 means no detection
int Commander::PlayerVisible(void)
{
	dbCompositionList list;
	g_database.ComposeList(list, OBJECT_Player);
	if(list[0]->GetStateMachine()->GetState() == PLYDEAD) return 0;
	D3DXVECTOR3 distance = g_terrain.m_player->GetBody().GetPos() - m_owner->GetBody().GetPos();
	D3DXVECTOR3 direction = distance / D3DXVec3Length(&distance);
	D3DXVECTOR3 myDir = m_owner->GetBody().GetDir();
	float dot = direction.x * myDir.x + direction.y * myDir.y + direction.z * myDir.z;
	float dist = D3DXVec3Length(&distance);
	if(g_terrain.LOSCheck(&m_owner->GetBody().GetPos(), &distance) == NULL && dot > 0)
	{
		if(dist <= 1.0f) return 2;
		else if(dist <= 1.4f) return 1;
	}
	return 0;
}
