#include "dxstdafx.h"
#include "Private.h"
#include "database.h"
#include "movement.h"
#include "body.h"
#include "terrain.h"

#define PLYDEAD 4

//Add new states here
enum StateName {
	STATE_Initialize,	//Note: the first enum is the starting state
	STATE_Patrol,
	STATE_Idle,
	STATE_Attacking,
	STATE_Radioing,
	STATE_Investigate,
	STATE_Dead,
	STATE_Engage
};

//Add new substates here
enum SubstateName {
	//empty
};

bool Private::States( State_Machine_Event event, MSG_Object * msg, int state, int substate )
{
BeginStateMachine

	//Global message responses
	OnMsg( MSG_Reset )
		ResetStateMachine();

	OnMsg( MSG_SetHeuristic )
		m_heuristicWeight = msg->GetFloatData();

	OnMsg( MSG_KillRadio )
		m_radio = false;

	OnMsg(MSG_Die)
		ChangeState(STATE_Dead);

	//Messages for the Private to Respond To
	OnMsg(MSG_StatusRequest)
		if(m_radio) 
		{
			m_owner->GetTiny().PlaySoundEffect(SOUND_OK);
			SendMsg(MSG_OK, msg->GetSender(), m_myIndex);
		}
		else 
		{
			m_owner->GetTiny().PlaySoundEffect(SOUND_NO_RESPONSE);
			SendMsg(MSG_NoResponse, msg->GetSender(), m_myIndex);
		}

	OnMsg(MSG_EnemySighted)
		if(m_radio && msg->IsDataValid() && g_terrain.m_player->GetStateMachine()->GetState() != PLYDEAD)
		{
			m_owner->GetTiny().PlaySoundEffect(SOUND_YESSIR);
			m_lastKnownSighting = ((RadioInformation*)(msg->GetIntData()))->location;
			delete (RadioInformation*)(msg->GetIntData());
			ChangeState(STATE_Engage);
		}

	OnMsg(MSG_Investigate)
		if(m_radio && msg->IsDataValid())
		{
			m_owner->GetTiny().PlaySoundEffect(SOUND_YESSIR);
			m_lastKnownSighting = ((RadioInformation*)(msg->GetIntData()))->location;
			delete (RadioInformation*)(msg->GetIntData());
			ChangeState(STATE_Investigate);
		}

	OnMsg(MSG_EnemyDead)
		if(m_radio)
		{
			//play sound
			m_owner->GetTiny().PlaySoundEffect(SOUND_YESSIR);
			ChangeState(STATE_Patrol);
		}

	OnMsg(MSG_ReturnToSurveillance)
		if(m_radio)
		{
			//play sound
			m_owner->GetTiny().PlaySoundEffect(SOUND_YESSIR);
			ChangeState(STATE_Patrol);
		}

	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Initialize )

		OnEnter
			m_owner->GetMovement().SetIdleSpeed();
			m_heuristicWeight = 1.0f;
			ChangeState(STATE_Idle);

	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Patrol )

		OnEnter
			m_owner->GetMovement().SetWalkSpeed();
			int r = 0, c = 0;
			if(m_patrolList.size() > 0)
			{
				
				g_terrain.GetRowColumn(&(m_patrolList[index]), &r, &c);
				m_owner->GetMovement().ComputePath( r, c, m_heuristicWeight);
			}
			else ChangeState(STATE_Idle);

		OnUpdate
			//do line of sight check
			if(PlayerVisible() == 2)
			{
				m_lastKnownSighting = g_terrain.m_player->GetBody().GetPos();
				curRadioMessage = ENEMY_SIGHTED;
				ChangeState(STATE_Radioing);
			}
			else if(PlayerVisible() == 1)
			{
				m_lastKnownSighting = g_terrain.m_player->GetBody().GetPos();
				m_owner->GetTiny().PlaySoundEffect(SOUND_INVESTIGATE);
				ChangeState(STATE_Investigate);
			}

		OnMsg(MSG_Arrived)
			index = (index >= m_patrolList.size() - 1)? 0: index+1;
			ChangeState(STATE_Idle);

		//OnMsg(MSG_StatusRequest)
		//OnMsg(MSG_OK)
		//OnMsg(MSG_NoResponse)
		//OnMsg(MSG_Investigate)
		//OnMsg(MSG_ReplaceSquad)
		//OnMsg(MSG_CallForReinforcements)
		//OnMsg(MSG_EnemySighted)
		//OnMsg(MSG_EnemyDead)
		//OnMsg(MSG_ReturnToSurveillance)
		//OnMsg(MSG_LeaderDead)

	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Idle )

		OnEnter
			m_owner->GetMovement().SetIdleSpeed();
			ChangeStateDelayed(5.0f, STATE_Patrol);

		OnUpdate
			if(PlayerVisible() == 2)
			{
				m_lastKnownSighting = g_terrain.m_player->GetBody().GetPos();
				curRadioMessage = ENEMY_SIGHTED;
				ChangeState(STATE_Radioing);
			}
			else if(PlayerVisible() == 1)
			{
				m_lastKnownSighting = g_terrain.m_player->GetBody().GetPos();
				m_owner->GetTiny().PlaySoundEffect(SOUND_INVESTIGATE);
				ChangeState(STATE_Investigate);
			}

		//OnMsg(MSG_StatusRequest)
		//OnMsg(MSG_OK)
		//OnMsg(MSG_NoResponse)
		//OnMsg(MSG_Investigate)
		//OnMsg(MSG_ReplaceSquad)
		//OnMsg(MSG_CallForReinforcements)
		//OnMsg(MSG_EnemySighted)
		//OnMsg(MSG_EnemyDead)
		//OnMsg(MSG_ReturnToSurveillance)
		//OnMsg(MSG_LeaderDead)

	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Attacking )
		/*
			It is presumed that m_lastKnownSighting has been filled with a current location
			of the enemy, as the commander will only attack when the enemy is sighted
			set goal to a location near the enemy, with a LOS
		*/
		OnEnter
			int r = 0, c = 0;
			g_terrain.GetRowColumn(&(m_lastKnownSighting), &r, &c);
			m_owner->GetMovement().ComputePath( r, c, m_heuristicWeight);
			m_owner->GetMovement().SetJogSpeed();
		OnUpdate
			dbCompositionList list;
			g_database.ComposeList(list, OBJECT_Player);
			int r, c, ar, ac;
			g_terrain.GetRowColumn(&(g_terrain.m_player->GetBody().GetPos()), &r, &c);
			g_terrain.GetRowColumn(&(m_owner->GetBody().GetPos()), &ar, &ac);
			//if the player has moved check if we can still see him
			if((m_owner->GetMovement().goalr != r || 
				m_owner->GetMovement().goalc != c) && list[0]->GetStateMachine()->GetState() != PLYDEAD)
			{
				//if we can still see him, then recompute the path to his new location
				D3DXVECTOR3 direction = g_terrain.m_player->GetBody().GetPos() - m_owner->GetBody().GetPos();
				if(g_terrain.LOSCheck(&m_owner->GetBody().GetPos(), &direction) == NULL)
				{
					m_lastKnownSighting = g_terrain.m_player->GetBody().GetPos();
					m_owner->GetMovement().ComputePath( r, c, m_heuristicWeight);
				}
			}
			if(ar == r && ac == c)
			{
				SendMsg(MSG_Die, g_terrain.m_player->GetID());
				curRadioMessage = ENEMY_DEAD;
				ChangeState(STATE_Radioing);
			}

		OnMsg(MSG_StatusRequest)
			/*if(m_radio)
			{
				m_owner->GetTiny().PlaySoundEffect(SOUND_ENEMY_SIGHTED);
				RadioInformation* info = new RadioInformation(m_lastKnownSighting, m_myIndex);
				SendMsgDelayed(2.0f, MSG_EnemySighted, m_squadCommander->GetID(), (int)info);
			}*/
		OnMsg(MSG_Investigate)
		OnMsg(MSG_EnemySighted)
		//OnMsg(MSG_EnemyDead)
		//OnMsg(MSG_ReturnToSurveillance)

		OnExit
			m_owner->GetMovement().SetWalkSpeed();

	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Radioing )
		OnEnter
			//go through list of squad leaders and radio each
			//after a message, the commander MUST change state
			//NOTE: We need to go over each case
			switch(curRadioMessage)
			{
				case NO_MESSAGE:
					assert(!1);
					break;

				case STATUS_REQUEST:
					if(m_radio && m_squadLeader) SendMsgDelayed(2.0f, MSG_StatusRequest, m_squadLeader->GetID());
					break;

				case OK:
					m_owner->GetTiny().PlaySoundEffect(SOUND_OK);
					if(m_radio && m_squadLeader) SendMsgDelayed(2.0f, MSG_OK, m_squadLeader->GetID());
					else if(m_radio && m_squadCommander) SendMsgDelayed(2.0f, MSG_OK, m_squadCommander->GetID());
					ChangeState(STATE_Patrol);
					break;

				case INVESTIGATE:
					if(m_radio && m_squadLeader) 
					{
						m_owner->GetTiny().PlaySoundEffect(SOUND_REQUEST_INVESTIGATE);
						RadioInformation* info = new RadioInformation(m_lastKnownSighting, m_myIndex);
						SendMsgDelayed(2.0f, MSG_Investigate, m_squadLeader->GetID(), (int)info);
					}
					else if(m_radio && m_squadCommander) 
					{
						m_owner->GetTiny().PlaySoundEffect(SOUND_REQUEST_INVESTIGATE);
						RadioInformation* info = new RadioInformation(m_lastKnownSighting, m_myIndex);
						SendMsgDelayed(2.0f, MSG_Investigate, m_squadCommander->GetID());
					}
					else m_owner->GetTiny().PlaySoundEffect(SOUND_INVESTIGATE);
					ChangeState(STATE_Investigate);
					break;

				case ENEMY_SIGHTED:
					if(m_radio && m_squadLeader) 
					{
						m_owner->GetTiny().PlaySoundEffect(SOUND_ENEMY_SIGHTED);
						RadioInformation* info = new RadioInformation(m_lastKnownSighting, m_myIndex);
						SendMsgDelayed(2.0f, MSG_EnemySighted, m_squadLeader->GetID(), (int)info);
					}
					else if(m_radio && m_squadCommander) 
					{
						m_owner->GetTiny().PlaySoundEffect(SOUND_ENEMY_SIGHTED);
						RadioInformation* info = new RadioInformation(m_lastKnownSighting, m_myIndex);
						SendMsgDelayed(2.0f, MSG_Investigate, m_squadCommander->GetID());
					}
					ChangeState(STATE_Attacking);
					break;

				case ENEMY_DEAD:
					m_owner->GetTiny().PlaySoundEffect(SOUND_ENEMY_DEAD);
					/*if(m_radio && m_squadLeader) SendMsgDelayed(2.0f, MSG_EnemyDead, m_squadLeader->GetID());
					else if(m_radio && m_squadCommander) SendMsgDelayed(2.0f, MSG_EnemyDead, m_squadCommander->GetID());*/
					if(m_radio && m_squadLeader && m_squadLeader->IsAlive()) SendMsg(MSG_EnemyDead, m_squadLeader->GetID());
					else if(m_radio && m_squadCommander && m_squadCommander->IsAlive()) SendMsg(MSG_EnemyDead, m_squadCommander->GetID());
					ChangeState(STATE_Patrol);
					break;

				case RETURN_TO_SURVEILLANCE:
					m_owner->GetTiny().PlaySoundEffect(SOUND_RETURN_TO_SURVEILLANCE);
					if(m_radio && m_squadLeader && m_squadLeader->IsAlive()) SendMsgDelayed(2.0f, MSG_ReturnToSurveillance, m_squadLeader->GetID());
					else if(m_radio && m_squadCommander && m_squadCommander->IsAlive()) SendMsgDelayed(2.0f, MSG_ReturnToSurveillance, m_squadCommander->GetID());
					ChangeState(STATE_Patrol);
					break;

				default:
					assert(!1);
					break;
			}
			//PopState();	//Should Never Get To Here!

	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Investigate )
	/*
		Private has seen or heard something, but isn't sure that it's the enemy
		The spot to investigate will be in m_lastKnownSighting
	*/
		OnEnter
			m_owner->GetMovement().SetWalkSpeed();
			int r, c;
			g_terrain.GetRowColumn(&m_lastKnownSighting, &r, &c);
			m_owner->GetMovement().ComputePath(r, c, 1.0f);

		OnUpdate
			//do los check
			if(PlayerVisible() == 2)
			{
				m_lastKnownSighting = g_terrain.m_player->GetBody().GetPos();
				curRadioMessage = ENEMY_SIGHTED;
				ChangeState(STATE_Radioing);
			}

		OnMsg(MSG_Arrived)
			curRadioMessage = OK;
			ChangeState(STATE_Radioing);

		OnMsg(MSG_StatusRequest)
		OnMsg(MSG_Investigate)
		//OnMsg(MSG_EnemySighted)
		//OnMsg(MSG_EnemyDead)
		//OnMsg(MSG_ReturnToSurveillance)

		//Must Write A Check For If The Commander
		//Is Investigating Non-Responsive Group

	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Engage )
	/*
		Private has seen or heard something, but isn't sure that it's the enemy
		The spot to investigate will be in m_lastKnownSighting
	*/
		OnEnter
			m_owner->GetMovement().SetJogSpeed();
			int r, c;
			g_terrain.GetRowColumn(&m_lastKnownSighting, &r, &c);
			m_owner->GetMovement().ComputePath(r, c, 1.0f);

		OnUpdate
			//do los check
			if(PlayerVisible() == 2)
			{
				m_lastKnownSighting = g_terrain.m_player->GetBody().GetPos();
				ChangeState(STATE_Attacking);
			}

		OnMsg(MSG_Arrived)
			curRadioMessage = OK;
			ChangeState(STATE_Radioing);

		OnMsg(MSG_StatusRequest)
		OnMsg(MSG_Investigate)
		OnMsg(MSG_EnemySighted)
		//OnMsg(MSG_EnemyDead)
		//OnMsg(MSG_ReturnToSurveillance)

	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Dead )
	/*
		We died.
	*/
		OnEnter
		OnUpdate
			if(m_owner->GetBody().GetPos().y > -0.5)
				m_owner->GetBody().GetPos().y-= 0.01;

		OnMsg(MSG_StatusRequest) 
			if(m_squadCommander) SendMsgDelayed(2.0f, MSG_NoResponse, m_squadCommander->GetID(), m_myIndex);
			m_owner->SetAlive(false);
		OnMsg(MSG_Investigate)
		OnMsg(MSG_EnemySighted)
		OnMsg(MSG_EnemyDead)
		OnMsg(MSG_ReturnToSurveillance)

EndStateMachine
}

//If the player is nearby, we are facing the player (180 degree FOV) and we can
//see him directly, then visible
//2 means direct sighting, 1 means he is somewhat detected, 0 means no detection
int Private::PlayerVisible(void)
{

	dbCompositionList list;
	g_database.ComposeList(list, OBJECT_Player);
	if(list[0]->GetStateMachine()->GetState() == PLYDEAD) return 0;
	D3DXVECTOR3 distance = g_terrain.m_player->GetBody().GetPos() - m_owner->GetBody().GetPos();
	D3DXVECTOR3 direction = distance / D3DXVec3Length(&distance);
	D3DXVECTOR3 myDir = m_owner->GetBody().GetDir();
	float dot = direction.x * myDir.x + direction.y * myDir.y + direction.z * myDir.z;
	float dist = D3DXVec3Length(&distance);
	if(g_terrain.LOSCheck(&m_owner->GetBody().GetPos(), &distance) == NULL && dot > 0)
	{
		if(dist <= 1.0f) return 2;
		else if(dist <= 1.4f) return 1;
	}
	return 0;
}
