//-----------------------------------------------------------------------------
// File: Tiny.h
//
// Desc: Header file for the CTiny class.  Its declaraction is found here.
//
// Copyright (c) Microsoft Corporation. All rights reserved
//-----------------------------------------------------------------------------


#ifndef TINY_H
#define TINY_H


#define IDLE_TRANSITION_TIME 0.125f
#define MOVE_TRANSITION_TIME 0.25f


typedef std::basic_string< TCHAR > String;




#define ANIMINDEX_FAIL				0xffffffff
#define SOUND_EFFECTS				51
#define NUM_DIFF_SOUNDS				3
#define FOOTFALLSOUND00				L"footfall00.wav"
#define FOOTFALLSOUND01				L"footfall01.wav"
#define FOOTFALLSOUND02				L"footfall01.wav"
#define STATUSREQUEST01				L"../Sounds/StatusRequest01.wav"	//3
#define STATUSREQUEST02				L"../Sounds/StatusRequest02.wav"
#define STATUSREQUEST03				L"../Sounds/StatusRequest03.wav"
#define OK01						L"../Sounds/OK01.wav"				//6
#define OK02						L"../Sounds/OK02.wav"
#define OK03						L"../Sounds/OK03.wav"
#define NORESPONSE01				L"../Sounds/Static1.wav"		//9
#define NORESPONSE02				L"../Sounds/Static2.wav"
#define NORESPONSE03				L"../Sounds/Static3.wav"
#define INVESTIGATE01				L"../Sounds/Investigate01.wav"		//12
#define INVESTIGATE02				L"../Sounds/Investigate02.wav"
#define INVESTIGATE03				L"../Sounds/Investigate03.wav"
#define REPLACESQUAD01				L"../Sounds/ReplaceSquad01.wav"		//15
#define REPLACESQUAD02				L"../Sounds/ReplaceSquad02.wav"
#define REPLACESQUAD03				L"../Sounds/ReplaceSquad03.wav"
#define CALLFORREINFORCEMENTS01		L"../Sounds/CallForReinforcements01.wav"	//18
#define CALLFORREINFORCEMENTS02		L"../Sounds/CallForReinforcements02.wav"
#define CALLFORREINFORCEMENTS03		L"../Sounds/CallForReinforcements03.wav"
#define ENEMYSIGHTED01				L"../Sounds/EnemySighted01.wav"		//21
#define ENEMYSIGHTED02				L"../Sounds/EnemySighted02.wav"
#define ENEMYSIGHTED03				L"../Sounds/EnemySighted03.wav"
#define ENEMYDEAD01					L"../Sounds/EnemyDead01.wav"		//24
#define ENEMYDEAD02					L"../Sounds/EnemyDead02.wav"
#define ENEMYDEAD03					L"../Sounds/EnemyDead03.wav"
#define RETURNTOSURVEILLANCE01		L"../Sounds/ReturnToSurveillance01.wav"		//27
#define RETURNTOSURVEILLANCE02		L"../Sounds/ReturnToSurveillance02.wav"
#define RETURNTOSURVEILLANCE03		L"../Sounds/ReturnToSurveillance03.wav"
#define LEADERDEAD01				L"../Sounds/LeaderDead01.wav"		//30
#define LEADERDEAD02				L"../Sounds/LeaderDead02.wav"
#define LEADERDEAD03				L"../Sounds/LeaderDead03.wav"
#define YESSIR01					L"../Sounds/Yessir01.wav"		//33
#define YESSIR02					L"../Sounds/Yessir02.wav"
#define YESSIR03					L"../Sounds/Yessir03.wav"
#define SELFINVESTIGATE01			L"../Sounds/SelfInvestigate01.wav"		//36
#define SELFINVESTIGATE02			L"../Sounds/SelfInvestigate02.wav"
#define SELFINVESTIGATE03			L"../Sounds/SelfInvestigate03.wav"
#define SENDINVESTIGATE01			L"../Sounds/SendInvestigate01.wav"		//39
#define SENDINVESTIGATE02			L"../Sounds/SendInvestigate02.wav"
#define SENDINVESTIGATE03			L"../Sounds/SendInvestigate03.wav"
#define SENDENGAGE01				L"../Sounds/SendEngage01.wav"		//42
#define SENDENGAGE02				L"../Sounds/SendEngage02.wav"
#define SENDENGAGE03				L"../Sounds/SendEngage03.wav"
#define SELFENGAGE01				L"../Sounds/SelfEngage01.wav"		//45
#define SELFENGAGE02				L"../Sounds/SelfEngage02.wav"
#define SELFENGAGE03				L"../Sounds/SelfEngage03.wav"
#define REQUESTINVESTIGATE01		L"../Sounds/RequestInvestigate01.wav"	//48
#define REQUESTINVESTIGATE02		L"../Sounds/RequestInvestigate02.wav"
#define REQUESTINVESTIGATE03		L"../Sounds/RequestInvestigate03.wav"


extern CSound * g_apSoundsTiny[ SOUND_EFFECTS ];
extern LPCWSTR soundFiles[SOUND_EFFECTS];
extern enum SOUNDS;




// Callback data for pass to the callback handler
struct CallbackDataTiny
{
    DWORD           m_dwFoot;       // Identify which foot caused the callback
    D3DXVECTOR3 *   m_pvCameraPos;  // Camera position when callback happened
    D3DXVECTOR3 *   m_pvTinyPos;    // Tiny's position when callback happened
};




//-----------------------------------------------------------------------------
// Name: class CBHandlerTiny
// Desc: Derived from ID3DXAnimationCallbackHandler.  Callback handler for
//       CTiny -- plays the footstep sounds
//-----------------------------------------------------------------------------
class CBHandlerTiny : public ID3DXAnimationCallbackHandler
{
    HRESULT CALLBACK HandleCallback( THIS_ UINT Track, LPVOID pCallbackData )
    {
        CallbackDataTiny * pCD = (CallbackDataTiny *) pCallbackData;

        // this is set to NULL if we're not playing sounds
        if( /*fornow*/ ! pCD || ! pCD->m_pvCameraPos )
            return S_OK;

        // scale volume by distance from tiny
        D3DXVECTOR3 vDiff;
        D3DXVec3Subtract( & vDiff, pCD->m_pvCameraPos, pCD->m_pvTinyPos );
        float fVolume = min( D3DXVec3LengthSq( & vDiff ), 1.f );
        fVolume *= -3000.f;

        // play the sound
        if( pCD && g_apSoundsTiny[ pCD->m_dwFoot ] )
            g_apSoundsTiny[ pCD->m_dwFoot ]->Play( 0, 0, (LONG) fVolume );

        return S_OK;
    }
};


class GameObject;


//-----------------------------------------------------------------------------
// Name: class CTiny
// Desc: This is the character class. It handles character behaviors and the
//       the associated animations.
//-----------------------------------------------------------------------------
class CTiny
{

protected:

    // -- data structuring
    CMultiAnim *         m_pMA;               // pointer to mesh-type-specific object
    DWORD                m_dwMultiAnimIdx;    // index identifying which CAnimInstance this object uses
    CAnimInstance *      m_pAI;               // pointer to CAnimInstance specific to this object
    std::vector<CTiny*> *m_pv_pChars;         // pointer to global array of CTiny* s
    CSoundManager *      m_pSM;               // pointer to sound management interface
    DWORD                m_dwAnimIdxLoiter,   // Indexes of various animation sets
                         m_dwAnimIdxWalk,
                         m_dwAnimIdxJog;
    CallbackDataTiny     m_CallbackData[ 2 ]; // Data to pass to callback handler

    // operational status
    double               m_dTimePrev;         // global time value before last update
    double               m_dTimeCurrent;      // current global time value
    bool                 m_bUserControl;      // true == user is controling character with the keyboard
    bool                 m_bPlaySounds;       // true == this instance is playing sounds
    DWORD                m_dwCurrentTrack;    // current animation track for primary animation

    // character traits
    float                m_fSpeedTurn;        // character's turning speed -- in radians/second
    ID3DXAnimationCallbackHandler *
                         m_pCallbackHandler;  // pointer to callback inteface to handle callback keys
    D3DXMATRIX           m_mxOrientation;     // transform that gets the mesh into a common world space
    float                m_fPersonalRadius;   // personal space radius -- things can't get closer than this
                                              // (note that no height information is given--not necessary for this sample)
    // character status
    //D3DXVECTOR3          m_vPos;              // current position in the map -- in our sample, y is always == 0
    D3DXVECTOR3          m_vPosTarget;        // This indicates where we are moving to
    float                m_fFacingTarget;     // The direction from Tiny's current position to the final destination
    float                m_fSpeedWalk;
    float                m_fSpeedJog;
    bool                 m_bIdle;             // set when Tiny is idling -- not turning toward a target
    bool                 m_bWaiting;          // set when Tiny is not idle, but cannot move
    double               m_dTimeIdling;       // countdown - Tiny Idles til this goes < 0
    double               m_dTimeWaiting;      // countdown - Tiny is waiting til this goes < 0, then picks a new target
    double               m_dTimeBlocked;      // countdown - Tiny must wait a small time when encountering a blocker before starting to walk again

    char                 m_szASName[64];      // Current track's animation set name (for preserving across device reset)

	GameObject*          m_owner;

public:

    CTiny( GameObject& owner );
    virtual ~CTiny();
    virtual HRESULT Setup( CMultiAnim *pMA, std::vector< CTiny* > *pv_pChars, CSoundManager *pSM, double dTimeCurrent );
    virtual void Cleanup();
    virtual CAnimInstance *GetAnimInstance();
    void GetPosition( D3DXVECTOR3 *pV );
    virtual HRESULT ResetTime();
    virtual HRESULT AdvanceTime( double dTimeDelta, D3DXVECTOR3 *pvEye );
    virtual HRESULT Draw();
    virtual void Report( std::vector < String > & v_sReport );
    virtual void SetUserControl();
    virtual bool IsUserControl() { return m_bUserControl; }
    virtual void SetSounds( bool bSounds );
    virtual void ChooseNewLocation( D3DXVECTOR3 *pV );
	inline GameObject* GetOwner( void )			{ return( m_owner ); }

	void SmoothLoiter();
    void SetMoveKey();
    void SetIdleKey( bool bResetPosition );
    double GetSpeedScale();

	void SetOrientation( void );
	void PlaySoundEffect(SOUNDS sound);

    HRESULT RestoreDeviceObjects( LPDIRECT3DDEVICE9 pd3dDevice );
    HRESULT InvalidateDeviceObjects();

protected:      // ************ The following are not callable by the app -- internal stuff

    bool IsBlockedByCharacter( D3DXVECTOR3 *pV );
    DWORD GetAnimIndex( char sString[] );
    HRESULT AddCallbackKeysAndCompress( LPD3DXANIMATIONCONTROLLER pAC, LPD3DXKEYFRAMEDANIMATIONSET pAS, DWORD dwNumCallbackKeys, D3DXKEY_CALLBACK aKeys[], DWORD dwCompressionFlags, FLOAT fCompression );
    HRESULT SetupCallbacksAndCompression();
    void SetIdleState();
    virtual bool IsOutOfBounds( D3DXVECTOR3 *pV );
};



#endif // #ifndef TINY_H
