/* Copyright Steve Rabin, 2005. 
 * All rights reserved worldwide.
 *
 * This software is provided "as is" without express or implied
 * warranties. You may freely copy and compile this source into
 * applications you distribute provided that the copyright text
 * below is included in the resulting source code, for example:
 * "Portions Copyright Steve Rabin, 2005"
 */


#include "dxstdafx.h"
#include "agent.h"
#include "database.h"
#include "movement.h"
#include "body.h"
#include "terrain.h"


//Add new states here
enum StateName {
	STATE_Initialize,	//Note: the first enum is the starting state
	STATE_ChooseTarget,
	STATE_MoveToGoal,
	STATE_Idle,
	STATE_Dead
};

//Add new substates here
enum SubstateName {
	//empty
};

bool Agent::States( State_Machine_Event event, MSG_Object * msg, int state, int substate )
{
BeginStateMachine

	//Global message responses
	OnMsg( MSG_Reset )
		ResetStateMachine();

	OnMsg( MSG_NextGoal )
		ChangeState( STATE_ChooseTarget );

	OnMsg( MSG_SetHeuristic )
		m_heuristicWeight = msg->GetFloatData();

	OnMsg(MSG_Die)
		ChangeState(STATE_Dead);

	OnMsg(MSG_Kill)
		//search for the enemy that we killed
		dbCompositionList npcList;
		unsigned int i;
		int row, col;
		g_database.ComposeList(npcList, OBJECT_NPC);
		for(i=0; i < npcList.size(); ++i)
		{
			g_terrain.GetRowColumn(&(npcList[i]->GetBody().GetPos()), &row, &col);
			if(m_owner->GetMovement().goalr == row && m_owner->GetMovement().goalc == col)
			{
				SendMsg(MSG_Die, npcList[i]->GetID());
			}
		}
		ChangeState(STATE_Idle);


	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Initialize )

		OnEnter
			deadFrameCounter=0;
			m_heuristicWeight = 1.0f;
			m_owner->GetMovement().SetIdleSpeed();


	///////////////////////////////////////////////////////////////
	DeclareState( STATE_ChooseTarget )

		OnEnter
			m_owner->GetMovement().SetJogSpeed();
			D3DXVECTOR3 goal( 0.0f, 0.0f, 0.0f );
			int r ,c;
			if(m_owner->GetMovement().m_usePickedGoal == false)
			{
				r = rand()%20;
				c = rand()%20;
				goal = g_terrain.GetCoordinates( r, c );
			}
			else
			{
				r = m_owner->GetMovement().goalr;
				c = m_owner->GetMovement().goalc;
				goal = g_terrain.GetCoordinates(r, c);
				m_owner->GetMovement().m_usePickedGoal = false;
			}
			/*if( !g_terrain.IsWall( r, c ) )
			{ 
				g_terrain.SetColor( r, c, DEBUG_COLOR_BLUE );
			}*/
			if(m_owner->GetMovement().ComputePath( r, c, m_heuristicWeight ) == true)
				ChangeState( STATE_MoveToGoal );
		OnUpdate
			int r,c;
			if(m_owner->GetMovement().m_started == true)
			{
				r=m_owner->GetMovement().m_astar->goal.x;
				c=m_owner->GetMovement().m_astar->goal.y;
				if(m_owner->GetMovement().ComputePath( r, c, m_heuristicWeight ) == true)
					ChangeState( STATE_MoveToGoal );
			}

	///////////////////////////////////////////////////////////////
	DeclareState( STATE_MoveToGoal )

		OnMsg( MSG_Arrived )
			/*BoxInfo* pBox = g_terrain.LOSCheck(&m_owner->GetBody().GetPos(), &m_owner->GetBody().GetDir());
			if(pBox != NULL)
			{
				pBox->loc.y += 1.0f/2.0f;
				//update the matrix for the box
				g_terrain.UpdateBoxMatrix(pBox);
			}*/
			ChangeState( STATE_Idle );


	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Idle )

		OnEnter
			m_owner->GetMovement().SetIdleSpeed();

	//////////////////////////////////////////////////////////////
	DeclareState( STATE_Dead )
		OnUpdate
			m_owner->GetMovement().SetTarget(m_owner->GetBody().GetPos());
			if(m_owner->GetBody().GetPos().y > -0.5)
				m_owner->GetBody().GetPos().y-= 0.01;
			if(deadFrameCounter >= DEADFRAMES)
			{
				deadFrameCounter=0;
				m_owner->GetBody().SetPos(D3DXVECTOR3(0.1f, 0.0f, 0.1f));
				SendMsgToMe(MSG_Reset);
			}
			else
				++deadFrameCounter;

EndStateMachine
}

