
#include "dxstdafx.h"
#include "database.h"
#include "movement.h"
#include "gameobject.h"
#include "body.h"
#include "tiny.h"
#include "debugdrawing.h"
#include "terrain.h"
#include <vector>


Movement::Movement( GameObject& owner )
: m_owner( &owner ),
  m_speedWalk( 1.f / 5.7f ),
  m_speedJog( 1.f / 2.3f ),
  m_movementMode( MOVEMENT_SEEK_TARGET ),
  m_useAStar(true), //CHANGE TO TRUE TO TEST ASTAR PATHFINDING
  m_step(false),
  m_usePickedGoal(false)
{
	m_target.x = m_target.y = m_target.z = 0.0f;
	if(m_useAStar == true)
	{
		//any intialization code here
		m_started = false;
	}
}

Movement::~Movement( void )
{

}

void Movement::Animate( double dTimeDelta )
{
	if( m_owner->GetBody().GetSpeed() == 0.0f )
	{
		m_owner->GetTiny().SmoothLoiter();
	}
	else
	{
		float speed = m_owner->GetBody().GetSpeed();
		D3DXVECTOR3 pos = m_owner->GetBody().GetPos();

		if( m_movementMode == MOVEMENT_SEEK_TARGET )
		{
			D3DXVECTOR3 toTarget = m_target - pos;

			if( D3DXVec3Length( &toTarget ) < 0.01f )
			{	//Notify target reached
				//m_owner->GetBody().SetPos( m_target );
				g_database.SendMsgFromSystem( m_owner, MSG_Arrived );
			}
			else
			{	
				//Point character towards target this frame
				D3DXVECTOR3 dir;
				D3DXVec3Normalize( &dir, &toTarget );
				m_owner->GetBody().SetDir( dir );

				//Move character towards target this frame
				double speedScale = m_owner->GetTiny().GetSpeedScale();
				D3DXVec3Scale( &dir, &dir, float( m_owner->GetBody().GetSpeed() * speedScale * dTimeDelta ) );

				D3DXVECTOR3 newPos;
				D3DXVec3Add( &newPos, &dir, &m_owner->GetBody().GetPos() );
				m_owner->GetBody().SetPos( newPos );
			}
		}
		else if( m_movementMode == MOVEMENT_WAYPOINT_LIST )
		{
			if(m_waypointList.empty() == false)
			{
				D3DXVECTOR3 target = m_waypointList.front();
				D3DXVECTOR3 toTarget = target - pos;

				float dist;
				if(m_waypointList.size() == 1) dist = 0.02f;
				else dist = 0.05f;

				if( D3DXVec3Length( &toTarget ) < dist )
				{	//Target reached - get new target
					m_waypointList.pop_front();
					if(m_waypointList.empty())
					{
						//We hit our goal
						g_database.SendMsgFromSystem( m_owner, MSG_Arrived );
						m_owner->GetTiny().SetOrientation();
						return;
					}
					target = m_waypointList.front();
					toTarget = target - pos;
				}

				//Point character towards target this frame
				D3DXVECTOR3 dir;
				D3DXVec3Normalize( &dir, &toTarget );
				m_owner->GetBody().SetDir( dir );

				//Move character towards target this frame
				double speedScale = m_owner->GetTiny().GetSpeedScale();
				D3DXVec3Scale( &dir, &dir, float( m_owner->GetBody().GetSpeed() * speedScale * dTimeDelta ) );

				D3DXVECTOR3 newPos;
				D3DXVec3Add( &newPos, &dir, &m_owner->GetBody().GetPos() );
				m_owner->GetBody().SetPos( newPos );
			}
		}
	}

	m_owner->GetTiny().SetOrientation();

}

void Movement::SetIdleSpeed( void )
{
	m_owner->GetBody().SetSpeed( 0.0f );
	m_owner->GetTiny().SetIdleKey( true );
}

void Movement::SetWalkSpeed( void )
{
	m_owner->GetBody().SetSpeed( m_speedWalk );
	m_owner->GetTiny().SetMoveKey();
}

void Movement::SetJogSpeed( void )
{
	m_owner->GetBody().SetSpeed( m_speedJog );
	m_owner->GetTiny().SetMoveKey();
}

bool Movement::ComputePath( int r, int c, float heuristicWeight )
{
	m_goal = g_terrain.GetCoordinates( r, c );
	m_movementMode = MOVEMENT_WAYPOINT_LIST;
	std::vector<Node> backList, forwardList;
	Node tempNode;
	D3DXVECTOR3 spot;
	int goalr, goalc, ret;
	if( m_useAStar )
	{
		m_waypointList.clear();
		Terrain *pTerrain = &g_terrain;
		g_terrain.GetRowColumn( &m_goal, &goalr, &goalc );
		if(g_terrain.IsWall(goalr, goalc) == false)
		{
			if(m_started == false)
			{
				m_astar = new AStar(m_owner->GetBody().GetPos(), m_goal, pTerrain, heuristicWeight);
				m_started = true;
				//if(CheckStraightLine() == true)
					//return(true);
			}
			if(m_step == false)
			{
				do{				
					ret = m_astar->Update();		
				}while(ret == 0);
			}
			else
			{
				ret = m_astar->Update();
				//paint the open and closed tiles
				ColorTiles(pTerrain, DEBUG_COLOR_GREEN, DEBUG_COLOR_RED);
			}
			if(ret == 1)
			{
				//path found
				//backList.push_back(m_astar->closedList[(int)(m_astar->closedList.size() - 1)]);
				std::list<Node>::iterator prevIter;
				for(std::list<Node>::iterator iter=m_astar->closedList.begin();
					iter != m_astar->closedList.end(); ++iter)
				{
					prevIter = iter; 
				}
				backList.push_back(*prevIter);
				
				if(backList[0].pos == m_astar->start)
				{
					//goal is the start
					spot = g_terrain.GetCoordinates( backList[0].pos.x, backList[0].pos.y );
					m_waypointList.push_back( spot );
				}
				else
				{
					//keep going until you get to the start
					
					tempNode = backList[0];
					while(tempNode.parent != NULL)
					{
						tempNode = *(tempNode.parent);
						backList.push_back(tempNode);
					}
					//now we have a list starting at the goal going to the start
					//puts the reverse of backList into forwardList
					std::vector<Node>::iterator iter=backList.end();
					--iter;
					while(iter != backList.begin())
					{
						forwardList.push_back( *iter );
						--iter;
					}
					//now push the goal onto the list
					forwardList.push_back( *iter );
					//now append these to tinys movement
					for(std::vector<Node>::iterator iter=forwardList.begin(); iter != forwardList.end(); ++iter)
					{
						spot = g_terrain.GetCoordinates( iter->pos.x, iter->pos.y );
						m_waypointList.push_back( spot );
					}
					SmoothCurrentWaypoints();
				}
				int curR, curC;
				D3DXVECTOR3 cur = m_waypointList.back();
				g_terrain.GetRowColumn( &cur, &curR, &curC );
				this->goalr = curR;
				this->goalc = curC;
			//delete the list
				delete m_astar;
				m_started=false;
				return(true);
			}
			else if(ret == -1)
			{
				//no path possible
				int breakpoint = 0;
				int curR, curC;
				D3DXVECTOR3 cur = m_owner->GetBody().GetPos();
				g_terrain.GetRowColumn( &cur, &curR, &curC );
				m_waypointList.push_back( cur );
				//delete the list
				delete m_astar;
				m_started=false;
				goalr = curR;
				goalc = curC;
				return(true);
			}
			else if(ret == 0)
			{
				int curR, curC;
				D3DXVECTOR3 cur = m_owner->GetBody().GetPos();
				g_terrain.GetRowColumn( &cur, &curR, &curC );
				m_waypointList.push_back( cur );
				goalr = curR;
				goalc = curC;
				return(false);
			}
		}//end of goal wall check
		else
		{
			int curR, curC;
			D3DXVECTOR3 cur = m_owner->GetBody().GetPos();
			g_terrain.GetRowColumn( &cur, &curR, &curC );
			m_waypointList.push_back( cur );
			goalr = curR;
			goalc = curC;
			return(true);
		}
	}
	else
	{	
		//Randomly meander toward goal (might get stuck at wall)
		int curR, curC;
		D3DXVECTOR3 cur = m_owner->GetBody().GetPos();
		g_terrain.GetRowColumn( &cur, &curR, &curC );

		m_waypointList.clear();
		m_waypointList.push_back( cur );

		int countdown = 100;
		while( curR != r || curC != c )
		{
			if( countdown-- < 0 ) { break; }

			if( curC == c || (curR != r && rand()%2 == 0) )
			{	//Go in row direction
				int last = curR;
				if( curR < r ) { curR++; }
				else { curR--; }

				if( g_terrain.IsWall( curR, curC ) )
				{
					curR = last;
					continue;
				}
			}
			else
			{	//Go in column direction
				int last = curC;
				if( curC < c ) { curC++; }
				else { curC--; }

				if( g_terrain.IsWall( curR, curC ) )
				{
					curC = last;
					continue;
				}
			}

			D3DXVECTOR3 spot = g_terrain.GetCoordinates( curR, curC );
			m_waypointList.push_back( spot );
		}
		SmoothCurrentWaypoints();
		return(true);
	}
}

void Movement::DrawDebugVisualization( IDirect3DDevice9* pd3dDevice, D3DXMATRIX* pViewProj )
{
	//Draw waypoints
	if( !m_waypointList.empty() )
	{
		D3DXVECTOR3 p0, p1;
		D3DXVECTOR3 last = *m_waypointList.begin();
		D3DXVECTOR3 cur = m_owner->GetBody().GetPos();
		p0 = last;
		cur.y = p0.y = 0.01f;
		g_debugdrawing.DrawLine( pd3dDevice, pViewProj, cur, p0, DEBUG_COLOR_RED, true );

		WaypointList::iterator i = m_waypointList.begin();
		for( WaypointList::iterator i = m_waypointList.begin(); i != m_waypointList.end(); i++ )
		{
			p0 = *i;
			p0.y = 0.01f;
			p1 = p0;
			p1.y += 0.03f;
			g_debugdrawing.DrawLine( pd3dDevice, pViewProj, p0, p1, DEBUG_COLOR_BLACK, false );
			g_debugdrawing.DrawLine( pd3dDevice, pViewProj, last, p0, DEBUG_COLOR_RED, true );
			last = p0;
		}
	}

	//Draw goal circle
	if( m_movementMode == MOVEMENT_WAYPOINT_LIST )
	{
		D3DXVECTOR3 cur = m_goal;
		cur.y = 0.006f;
		g_debugdrawing.DrawCircle( pd3dDevice, pViewProj, &cur, 0.005f, DEBUG_COLOR_WHITE );
		g_debugdrawing.DrawCircle( pd3dDevice, pViewProj, &cur, 0.010f, DEBUG_COLOR_BLACK );
		g_debugdrawing.DrawCircle( pd3dDevice, pViewProj, &cur, 0.015f, DEBUG_COLOR_WHITE );
		g_debugdrawing.DrawCircle( pd3dDevice, pViewProj, &cur, 0.020f, DEBUG_COLOR_BLACK );
		g_debugdrawing.DrawCircle( pd3dDevice, pViewProj, &cur, 0.025f, DEBUG_COLOR_WHITE );
	}
}

void Movement::SmoothCurrentWaypoints( void )
{
	if( m_waypointList.size() > 2 )
	{
		D3DXVECTOR3 newPoint0( 0.0f, 0.0f, 0.0f );
		D3DXVECTOR3 newPoint1( 0.0f, 0.0f, 0.0f );

		WaypointList oldList;
		for( WaypointList::iterator i = m_waypointList.begin(); i != m_waypointList.end(); i++ )
		{
			oldList.push_back( *i );
		}
		m_waypointList.clear();

		D3DXVECTOR3 p0, p1, p2, p3;

		//First
		WaypointList::iterator iter = oldList.begin();
		p0 = p1 = p2 = *iter;
		iter++;
		p3 = *iter;
		iter++;

		const int divisions = 5;
		while( iter != oldList.end() )
		{
			p0 = p1;
			p1 = p2;
			p2 = p3;
			p3 = *iter;

			for( int count=1; count<divisions+1; count++)
			{
				D3DXVECTOR3 newPoint = CatmullRom( p0, p1, p2, p3, (float)count/(float)divisions );
				if( m_waypointList.size() > 1 )
				{
					D3DXVECTOR3 lastDir = newPoint1 - newPoint0;
					D3DXVECTOR3 curDir = newPoint - newPoint1;
					D3DXVec3Normalize( &lastDir, &lastDir );
					D3DXVec3Normalize( &curDir, &curDir );
					float dot = D3DXVec3Dot( &lastDir, &curDir );
					if( dot > 0.99f )
					{
						m_waypointList.pop_back();
					}
				}
				newPoint0 = newPoint1;
				newPoint1 = newPoint;
				m_waypointList.push_back( newPoint );
			}
			iter++;
		}

		//Last
		p0 = p1;
		p1 = p2;
		p2 = p3;
		p3 = p3;	//Redundant, but put here to make it clear what is intended
		
		for( int count=1; count<divisions+1; count++)
		{
			D3DXVECTOR3 newPoint = CatmullRom( p0, p1, p2, p3, (float)count/(float)divisions );
			if( m_waypointList.size() > 1 )
			{
				D3DXVECTOR3 lastDir = newPoint1 - newPoint0;
				D3DXVECTOR3 curDir = newPoint - newPoint1;
				D3DXVec3Normalize( &lastDir, &lastDir );
				D3DXVec3Normalize( &curDir, &curDir );
				float dot = D3DXVec3Dot( &lastDir, &curDir );
				if( dot > 0.99f )
				{
					m_waypointList.pop_back();
				}
			}
			newPoint0 = newPoint1;
			newPoint1 = newPoint;
			m_waypointList.push_back( newPoint );
		}
	}
}

D3DXVECTOR3 Movement::CatmullRom( D3DXVECTOR3 p0, D3DXVECTOR3 p1, D3DXVECTOR3 p2, D3DXVECTOR3 p3, float u )
{
	float u_squared = u*u;
	float u_cubed = u_squared*u;

	D3DXVECTOR3 result = p0 * (-0.5f*u_cubed + u_squared - 0.5f*u) +
		                 p1 * (1.5f*u_cubed - 2.5f*u_squared + 1.0f) +
						 p2 * (-1.5f*u_cubed + 2.0f*u_squared + 0.5f*u) +
						 p3 * (0.5f*u_cubed - 0.5f*u_squared);

	return( result );
}

void Movement::ColorTiles(Terrain *t, DebugDrawingColor openColor, DebugDrawingColor closedColor)
{
	t->ResetColors();
	for(std::list<Node>::iterator iter = m_astar->openList.begin(); iter != m_astar->openList.end(); ++iter)
	{
		t->SetColor(iter->pos.x, iter->pos.y, openColor);
	}
	for(std::list<Node>::iterator iter = m_astar->closedList.begin(); iter != m_astar->closedList.end(); ++iter)
	{
		t->SetColor(iter->pos.x, iter->pos.y, closedColor);
	}
}
bool Movement::CheckStraightLine()
{
	D3DXVECTOR3 cur;
	int startr, startc, goalr, goalc, diff, tempSpot;
	startr = m_astar->start.x;
	startc = m_astar->start.y;
	goalr = m_astar->goal.x;
	goalc = m_astar->goal.y;
	if(startr == goalr)
	{
		tempSpot = startc;
		diff = goalc - startc;
		if(diff < 0)
		{
			//subtract one every time
			while(tempSpot >= goalc)
			{
				cur = g_terrain.GetCoordinates(startr, tempSpot);
				m_waypointList.push_back( cur );
				--tempSpot;
			}
		}
		else
		{
			while(tempSpot <= goalc)
			{
				cur = g_terrain.GetCoordinates(startr, tempSpot);
				m_waypointList.push_back( cur );
				++tempSpot;
			}
		}
		return(true);
	}
	else if(startc == goalc)
	{
		tempSpot = startr;
		diff = goalr - startr;
		if(diff < 0)
		{
			//subtract one every time
			while(tempSpot >= goalr)
			{
				cur = g_terrain.GetCoordinates(startr, tempSpot);
				m_waypointList.push_back( cur );
				--tempSpot;
			}
		}
		else
		{
			while(tempSpot <= goalr)
			{
				cur = g_terrain.GetCoordinates(startr, tempSpot);
				m_waypointList.push_back( cur );
				++tempSpot;
			}
		}
		return(true);
	}
	return(false);
}