#include "astar.h"

#pragma once

class GameObject;

typedef std::list<D3DXVECTOR3> WaypointList;

enum MovementMode
{
	MOVEMENT_NULL,
	MOVEMENT_SEEK_TARGET,
	MOVEMENT_WAYPOINT_LIST
};

class Movement
{
public:
	Movement( GameObject& owner );
	~Movement( void );

	inline void SetTarget( D3DXVECTOR3& target )			{ m_target = target; m_movementMode = MOVEMENT_SEEK_TARGET; }
	inline D3DXVECTOR3& GetTarget( void )					{ return( m_target ); }

	void Animate( double dTimeDelta );
	void DrawDebugVisualization( IDirect3DDevice9* pd3dDevice, D3DXMATRIX* pViewProj );

	void SetIdleSpeed( void );
	void SetWalkSpeed( void );
	void SetJogSpeed( void );

	bool ComputePath( int r, int c, float heuristicWeight );
	void SmoothCurrentWaypoints( void );

	void ColorTiles(Terrain *t, DebugDrawingColor openColor, DebugDrawingColor closedColor);
	bool CheckStraightLine();
	D3DXVECTOR3 CatmullRom( D3DXVECTOR3 p0, D3DXVECTOR3 p1, D3DXVECTOR3 p2, D3DXVECTOR3 p3, float u );
	bool m_started;
	AStar *m_astar;
	bool m_usePickedGoal;
	int goalr, goalc;

protected:

	GameObject* m_owner;
	
	
	bool m_useAStar;
	bool m_step;

	MovementMode m_movementMode;

	D3DXVECTOR3 m_target;
	D3DXVECTOR3 m_goal;

	WaypointList m_waypointList;

	float m_speedWalk;
	float m_speedJog;

};