

#include "dxstdafx.h"
#include "terrain.h"
#include "global.h"

#define NUM_RAND_MAPS 13
#define MAT_MAP 1

int g_seedMapValues[NUM_RAND_MAPS] = { 74, 54, 23, 34, 2, 934, 343, 99, 17, 3435, 343, 298, 888 };


Terrain::Terrain( void )
: m_nextMap(-1)
{
	pd3dDevice = NULL;
}

Terrain::~Terrain()
{
	//pd3dDevice = NULL;
	//SAFE_RELEASE(pd3dDevice);
	std::vector<BoxInfo>::iterator iter;
	for(iter = g_terrain.boxMeshes.begin();
		iter != g_terrain.boxMeshes.end(); ++iter)
	{
		(*iter).pMesh->Release();
	}
}

void Terrain::Create( IDirect3DDevice9* p )
{
	pd3dDevice = p;
	NextMap();
}

void Terrain::NextMap( void )
{
	BoxInfo tempBox;
	D3DXMATRIX m_Translation, m_scale;
	boxMeshes.clear();
	if( m_nextMap > NUM_RAND_MAPS )
	{
		m_nextMap = 0;
	}

	if( m_nextMap == NUM_RAND_MAPS )
	{	//Clear map
		for( int r=0; r<TERRAIN_WIDTH; r++ )
		{
			for( int c=0; c<TERRAIN_WIDTH; c++ )
			{
				m_terrain[r][c] = 0;
			}
		}
	}
	else
	{
		int modValue = 3;
		srand(g_seedMapValues[m_nextMap]);
		if( m_nextMap >= 10 ) { modValue = 10; }
#ifdef MAT_MAP
		for( int r=0; r<TERRAIN_WIDTH; r++ )
		{
			for( int c=0; c<TERRAIN_WIDTH; c++ )
			{
				m_terrain[r][c] = 0;
				if(r >= 75)
				{
					if(c == 50) m_terrain[r][c] = -1;
				}
				if(r == 75)
				{
					if(c >= 50 && c < 70) m_terrain[r][c] = -1;
				}
				if(r == 70)
				{
					if(c >= 60) m_terrain[r][c] = -1;
				}
				if(r < 70 && r >= 60)
				{
					if(c == 5) m_terrain[r][c] = -1;
					if(c == 15) m_terrain[r][c] = -1;
				}
				if(r == 60)
				{
					if(c >= 15 && c <= 75) m_terrain[r][c] = -1;
				}
				if(r < 60 && r >= 50)
				{
					if(c == 60) m_terrain[r][c] = -1;
					if(c == 75) m_terrain[r][c] = -1;
				}
				if(r == 55)
				{
					if(c <= 10) m_terrain[r][c] = -1;
				}
				if(r == 50)
				{
					if(c >= 60 && c <= 75) m_terrain[r][c] = -1;
				}
				if(r < 60 && r >= 45)
				{
					if(c == 25) m_terrain[r][c] = -1;
				}
				if(r < 55 && r >= 40)
				{
					if(c == 30) m_terrain[r][c] = -1;
				}
				if(r < 55 && r >= 45)
				{
					if(c == 10) m_terrain[r][c] = -1;
					if(c == 40) m_terrain[r][c] = -1;
				}
				if(r == 45)
				{
					if(c <= 10) m_terrain[r][c] = -1;
					if(c >= 40 && c < 55) m_terrain[r][c] = -1;
				}
				if(r < 40 && r >= 30)
				{
					if(c == 10) m_terrain[r][c] = -1;
					if(c == 20) m_terrain[r][c] = -1;
					if(c == 40) m_terrain[r][c] = -1;
				}
				if(r == 40)
				{
					if(c >= 10 && c <= 20) m_terrain[r][c] = -1;
					if(c > 20 && c <= 40) m_terrain[r][c] = -1;
				}

				if(r == 30)
				{
					if(c >= 20 && c <= 40) m_terrain[r][c] = -1;
				}
				if(r == 5)
				{
					if(c <= 40) m_terrain[r][c] = -1;
					if(c >= 50 && c <= 70) m_terrain[r][c] = -1;
				}
			}
		}
#else
		for( int r=0; r<TERRAIN_WIDTH; r++ )
		{
			for( int c=0; c<TERRAIN_WIDTH; c++ )
			{
				if( rand()%modValue == 0 )
				{
					m_terrain[r][c] = -1;
				}
				else
				{
					m_terrain[r][c] = 0;
				}
			}
		}
#endif
	}
	for( int r=0; r<TERRAIN_WIDTH; r++ )
	{
		for( int c=0; c<TERRAIN_WIDTH; c++ )
		{
			//if(false)
			if(m_terrain[r][c] < 0)
			{
				//We have a wall
				D3DXCreateBox(pd3dDevice, 1.1f, 1.1f, 1.1f, &tempBox.pMesh, NULL);
				tempBox.loc = GetCoordinates(r, c);
				//now make the world transform for the matrix
					/*for(int i=0; i < 4; ++i)
					{
						for(int j=0; j < 4; ++j)
						{
							if(i == j)
								tempMatrix.m[i][j] = 1;
							else
								tempMatrix.m[i][j] = 0;
						}
					}
					tempMatrix.m[3][0] = 0;//(*iter).loc.x;
					tempMatrix.m[3][1] = 0;//(*iter).loc.y;
					tempMatrix.m[3][2] = 0;//(*iter).loc.z;*/
				float ft = 1.0f/20.0f;
				D3DXMatrixScaling(&m_scale, ft, 0.25f, ft);
				D3DXMatrixTranslation(&m_Translation, tempBox.loc.x, tempBox.loc.y + .125f, tempBox.loc.z);
				D3DXMatrixMultiply(&tempBox.worldTrans, &m_scale, &m_Translation);
				boxMeshes.push_back(tempBox);
			}
		}
	}
	ResetColors();
	m_nextMap++;
}

D3DXVECTOR3 Terrain::GetCoordinates( int r, int c )
{
	D3DXVECTOR3 pos;
	pos.x = (((float)c * WORLD_SIZE)/(float)TERRAIN_WIDTH) + (WORLD_SIZE / ((float)TERRAIN_WIDTH * 2));
	pos.y = 0.0f;
	pos.z = (((float)r * WORLD_SIZE)/(float)TERRAIN_WIDTH) + (WORLD_SIZE / ((float)TERRAIN_WIDTH * 2));
	
	return( pos );
}

void Terrain::GetRowColumn( D3DXVECTOR3* pos, int* r, int* c )
{
	if( pos->x >= 0.0f && pos->x <= WORLD_SIZE && pos->z >= 0.0f && pos->z <= WORLD_SIZE )
	{
		*r = (int)((pos->z / WORLD_SIZE) * (float)TERRAIN_WIDTH);
		*c = (int)((pos->x / WORLD_SIZE) * (float)TERRAIN_WIDTH);
	}
	else
	{
		ASSERTMSG( 0, "Position is out of bounds of the grid - can't compute row/column" );
	}
}

void Terrain::SetColor( int r, int c, DebugDrawingColor color )
{
	if( m_terrain[r][c] >= 0 )
	{
		m_terrainColor[r][c] = color;
	}
	/*else
	{
		ASSERTMSG( 0, "Can't set color to a grid square that is a wall" );
	}*/
}

void Terrain::ResetColors( void )
{
	for( int r=0; r<TERRAIN_WIDTH; r++ )
	{
		for( int c=0; c<TERRAIN_WIDTH; c++ )
		{
			if( m_terrain[r][c] >= 0 )
			{
				m_terrainColor[r][c] = DEBUG_COLOR_WHITE;
			}
			else
			{
				m_terrainColor[r][c] = DEBUG_COLOR_BLACK;
			}
		}
	}
}

void Terrain::DrawDebugVisualization( IDirect3DDevice9* pd3dDevice )
{
	D3DXVECTOR3 pos( 0.0f, 0.005f, 0.0f );

	for( int r=0; r<TERRAIN_WIDTH; r++ )
	{
		for( int c=0; c<TERRAIN_WIDTH; c++ )
		{
			pos.x = ((float)c * WORLD_SIZE) / (float)TERRAIN_WIDTH;
			pos.z = ((float)r * WORLD_SIZE) / (float)TERRAIN_WIDTH;
			g_debugdrawing.DrawQuad( pd3dDevice, &pos,  WORLD_SIZE /(float)TERRAIN_WIDTH, m_terrainColor[r][c] );
		}
	}
}
/*
	Returns the first mesh collided by the ray, or NULL if nothing is hit
*/
BoxInfo* Terrain::LOSCheck(D3DXVECTOR3* pos, D3DXVECTOR3* direction)
{
	BOOL hit;
	std::vector<BoxInfo>::iterator iter;
	//epsilon so that the ray is not on the ground
	pos->y += 0.012f;	//BAD MATT!
	BoxInfo* returnBox = NULL;
	LPDIRECT3DVERTEXBUFFER9 pVB;
    LPDIRECT3DINDEXBUFFER9  pIB;
	WORD* pIndices;
    D3DVERTEX* pVertices;
	D3DXVECTOR3 v0, v1, v2;
	D3DXVECTOR4 four;
	float distance, pu, pv;
	float curDist = Dist(&(m_player->GetBody().GetPos()), pos);

	*direction = *direction * 10000;

	for(iter = g_terrain.boxMeshes.begin();
		iter != g_terrain.boxMeshes.end(); ++iter)
	{
		//D3DXVec3Unproject(&newpos, pos, &viewport, &projmat, &viewmat, &iter->worldTrans);
		//D3DXVec3Unproject(&newdir, direction, &viewport, &projmat, &viewmat, &iter->worldTrans);
		//D3DXIntersect((*iter).pMesh, &newpos, &newdir, &hit, &faceIndex, &pu, &pv, &dist, &pAllHits, &hitcount);
		//D3DXIntersect(&tempMesh, &newpos, &newdir, &hit, &faceIndex, &pu, &pv, &dist, &pAllHits, &hitcount);
		//D3DXIntersect(tempMesh, pos, direction, &hit, &faceIndex, &pu, &pv, &dist, NULL, NULL);

		//now will try to do it by checking every triangle
        iter->pMesh->GetVertexBuffer( &pVB );
        iter->pMesh->GetIndexBuffer( &pIB );

        pIB->Lock( 0, 0, (void**)&pIndices, 0 );
        pVB->Lock( 0, 0, (void**)&pVertices, 0 );
		//now make each triangle
		DWORD dwNumFaces = iter->pMesh->GetNumFaces();
		BOOL bhit;
		hit=0;
        for( DWORD i=0; i < (dwNumFaces - 1); i++ )
        {
            v0 = pVertices[pIndices[3*i+0]].p;
			D3DXVec3Transform(&four, &v0, &iter->worldTrans);
			v0.x = four.x;
			v0.y = four.y;
			v0.z = four.z;
            v1 = pVertices[pIndices[3*i+1]].p;
			D3DXVec3Transform(&four, &v1, &iter->worldTrans);
			v1.x = four.x;
			v1.y = four.y;
			v1.z = four.z;
            v2 = pVertices[pIndices[3*i+2]].p;
			D3DXVec3Transform(&four, &v2, &iter->worldTrans);
			v2.x = four.x;
			v2.y = four.y;
			v2.z = four.z;
			bhit=D3DXIntersectTri(&v0, &v1, &v2, pos, direction, &pu, &pv, &distance);
			if(bhit)
			{
				D3DXVECTOR3 hitPos = v0 + (pu * (v1 - v0)) + (pv *(v2 - v0));
				distance = Dist(pos, &hitPos);
				if(distance < curDist)
				{
					returnBox = &(*iter);
					curDist = distance;
				}
			}
		}
		pVB->Unlock();
        pIB->Unlock();
	}
	pos->y -= 0.012f;	//GOOD DANTE!
	if(returnBox != NULL)
		return(returnBox);
	return(NULL);
}
void Terrain::UpdateBoxMatrix(BoxInfo* curBox)
{
	D3DXMATRIX m_Translation, m_scale;
	float ft = 1.0f/20.0f;
	D3DXMatrixScaling(&m_scale, ft, 0.25f, ft);
	D3DXMatrixTranslation(&m_Translation, curBox->loc.x, curBox->loc.y + .125f, curBox->loc.z);
	D3DXMatrixMultiply(&(curBox->worldTrans), &m_scale, &m_Translation);
}
float Terrain::Dist(D3DXVECTOR3* p1, D3DXVECTOR3* p2)
{
	float tx, ty, tz;
	tx = pow(p1->x - p2->x, 2);
	ty = pow(p1->y - p2->y, 2);
	tz = pow(p1->z - p2->z, 2);
	return(sqrt(tx+ty+tz));
}
