/* Copyright Steve Rabin, 2005. 
 * All rights reserved worldwide.
 *
 * This software is provided "as is" without express or implied
 * warranties. You may freely copy and compile this source into
 * applications you distribute provided that the copyright text
 * below is included in the resulting source code, for example:
 * "Portions Copyright Steve Rabin, 2005"
 */

#pragma once

#include "singleton.h"
#include "debugdrawing.h"
#include "gameobject.h"
#include "body.h"
#include <D3dx9mesh.h>
#include <vector>

#define TERRAIN_WIDTH 100
#define WORLD_SIZE 4.0f

struct BoxInfo
{
	ID3DXMesh *pMesh;
	D3DXVECTOR3 loc;
	D3DXMATRIX worldTrans;
};
struct D3DVERTEX
{
    D3DXVECTOR3 p;
    D3DXVECTOR3 n;
    FLOAT       tu, tv;

    static const DWORD FVF;
};

class Terrain : public Singleton <Terrain>
{
public:

	Terrain( void );
	~Terrain( void );

	void Create( IDirect3DDevice9* p );
	void NextMap( void );

	D3DXVECTOR3 GetCoordinates( int r, int c );
	void GetRowColumn( D3DXVECTOR3* pos, int* r, int* c );
	inline bool IsWall( int r, int c )		{ return( m_terrain[r][c] < 0 ); }

	void SetColor( int r, int c, DebugDrawingColor color );
	void ResetColors( void );

	void DrawDebugVisualization( IDirect3DDevice9* pd3dDevice );
	BoxInfo* LOSCheck(D3DXVECTOR3* pos, D3DXVECTOR3* direction);
	void UpdateBoxMatrix(BoxInfo* curBox);
	float Dist(D3DXVECTOR3* p1, D3DXVECTOR3* p2);
	IDirect3DDevice9* pd3dDevice;
	std::vector<BoxInfo> boxMeshes;

	GameObject* m_player;

protected:
	int m_nextMap;
	float m_terrain[TERRAIN_WIDTH][TERRAIN_WIDTH];
	DebugDrawingColor m_terrainColor[TERRAIN_WIDTH][TERRAIN_WIDTH];

};
