/* Copyright Steve Rabin, 2005. 
 * All rights reserved worldwide.
 *
 * This software is provided "as is" without express or implied
 * warranties. You may freely copy and compile this source into
 * applications you distribute provided that the copyright text
 * below is included in the resulting source code, for example:
 * "Portions Copyright Steve Rabin, 2005"
 */

#include "dxstdafx.h"
#include "unittest1.h"


//Add new states here
enum StateName {
	STATE_Initialize,	//Note: the first enum is the starting state
	STATE_Chain1,
	STATE_Chain2,
	STATE_Chain3,
	STATE_Chain4,
	STATE_Chain5,
	STATE_Chain6,
	STATE_Chain7,
	STATE_Chain8,
	STATE_Chain9,
	STATE_Chain10,
	STATE_Chain11,
	STATE_Chain12,
	STATE_Chain13,
	STATE_Success,
	STATE_Failure
};

//Add new substates here
enum SubstateName {
	SUBSTATE_Inside1,
	SUBSTATE_Inside2
};

//unittest1 covers:
//1. BeginStateMachine, EndStateMachine
//2. DeclareState, DeclareSubstate
//3. OnMsg
//4. OnEnter, OnExit
//5. OnUpdate
//6. ChangeState, ChangeStateDelayed
//7. ChangeSubstate, ChangeSubstateDelayed
//8. SetTimer
//9. SendMsgDelayedToMe
//10. SendMsgToMe
//11. PopState


bool UnitTest1::States( State_Machine_Event event, MSG_Object * msg, int state, int substate )
{
BeginStateMachine

	//Global message responses
	OnMsg( MSG_UnitTestMessage )
		ChangeState( STATE_Chain4 );
	

	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Initialize )

		OnEnter
			ChangeStateDelayed( RandDelay( 1.0f, 3.0f ), STATE_Chain1 );


	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Chain1 )

		OnEnter
			ChangeStateDelayed( 1.0f, STATE_Chain2 );
			ChangeStateDelayed( 1.5f, STATE_Chain1 );	//Should be ignored when state changes because of scoping


	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Chain2 )

		OnEnter
			SendMsgDelayedToMe( 1.0f, MSG_UnitTestMessage );

		OnMsg( MSG_UnitTestMessage )
			ChangeState( STATE_Chain3 );


	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Chain3 )

		OnEnter
		SendMsgDelayedToMe( 1.0f, MSG_UnitTestMessage );	//Will get caught by the global message response


	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Chain4 )

		OnEnter
			m_count = 0;
			SetTimer( 1.18f, MSG_UnitTestTimer );

		OnMsg( MSG_UnitTestTimer )
			m_count++;
			if( m_count == 5 )
			{
				ChangeState( STATE_Chain5 );
			}


	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Chain5 )

		OnEnter
			ChangeSubstate( SUBSTATE_Inside1 );

		OnMsg( MSG_UnitTestTimer )
			ASSERTMSG( 0, "Shouldn't get here. The timer from the previous state should get killed due to scoping." );

		DeclareSubstate( SUBSTATE_Inside1 )
			OnEnter
				ChangeSubstateDelayed( 1.0f, SUBSTATE_Inside2 );

		DeclareSubstate( SUBSTATE_Inside2 )
			OnEnter
				ChangeStateDelayed( 1.0f, STATE_Chain6 );


	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Chain6 )
		
		OnEnter
			SendMsgDelayedToMe( 3.0f, MSG_UnitTestMessage, NO_SCOPING );	//Message will be sent regardless of current state
			ChangeStateDelayed( 1.0f, STATE_Chain7 );

		OnMsg( MSG_UnitTestMessage )
			ChangeState( STATE_Chain8 );


	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Chain7 )
		
		OnEnter
			SendMsgDelayedToMe( 1.0f, MSG_UnitTestMessage2 );

		OnMsg( MSG_UnitTestMessage2 )
			PopState();


	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Chain8 )
		
		OnEnter
			ChangeState( STATE_Chain9 );
		
		OnExit
			SendMsgDelayedToMe( 1.0f, MSG_UnitTestMessage, NO_SCOPING );


	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Chain9 )
		
		OnMsg( MSG_UnitTestMessage )
			ChangeState( STATE_Chain10 );


	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Chain10 )
		
		OnUpdate
			ChangeState( STATE_Chain11 );


	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Chain11 )
		
		OnEnter
			SendMsgToMe( MSG_UnitTestMessage );

		OnMsg( MSG_UnitTestMessage );
			ChangeState( STATE_Chain12 );


	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Chain12 )
		
		OnEnter
			SendMsgDelayedToMe( 1.0f, MSG_UnitTestMessage, SCOPE_TO_STATE, 3 );
			SendMsgDelayedToMe( 1.0f, MSG_UnitTestMessage, SCOPE_TO_STATE, 2 );
			SendMsgDelayedToMe( 1.0f, MSG_UnitTestMessage, SCOPE_TO_STATE, 1 );
		
		OnMsg( MSG_UnitTestMessage );
		if( msg->GetIntData() == 2)
		{
			ChangeState( STATE_Chain13 );
		}


	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Chain13 )
		
		OnEnter
			SendMsgToMe( MSG_UnitTestMessage );
			ChangeState( STATE_Success );

		OnMsg( MSG_UnitTestMessage );
			ChangeState( STATE_Failure );

		
	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Success )
		
		OnEnter
			OutputDebugString( L"UnitTest1 Success\n" );

		
	///////////////////////////////////////////////////////////////
	DeclareState( STATE_Failure )
		
		OnEnter
			OutputDebugString( L"UnitTest1 Failure\n" );


EndStateMachine
}
