


#include "dxstdafx.h"
#include "world.h"
#include "time.h"
#include "database.h"
#include "msgroute.h"
#include "movement.h"
#include "debuglog.h"
#include "debugdrawing.h"
#include "terrain.h"
#include "MultiAnimation.h"
#include "Tiny.h"
#include "body.h"

// State machines
#include "agent.h"
#include "Private.h"
#include "Commander.h"
#include "Leader.h"

// Unit test state machines
#include "unittest1.h"
#include "unittest2a.h"
#include "unittest2b.h"
#include "unittest2c.h"
#include "unittest3a.h"
#include "unittest3b.h"
#include "unittest4.h"

//#define UNIT_TESTING

extern POINT mouseCoords;
World::World(void)
: m_initialized(false)
{

}

World::~World(void)
{
//	delete m_time;
//	delete m_database;
//	delete m_msgroute;
//	delete m_debuglog;
//	delete m_debugdrawing;
	SAFE_DELETE(m_terrain);
}

void World::InitializeSingletons( void )
{
	//Create Singletons
	Time* m_time = new Time();
	Database* m_database = new Database();
	MsgRoute* m_msgroute = new MsgRoute();
	DebugLog* m_debuglog = new DebugLog();
	DebugDrawing* m_debugdrawing = new DebugDrawing();
	Terrain* m_terrain = new Terrain();
}

void World::Initialize( CMultiAnim *pMA, std::vector< CTiny* > *pv_pChars, CSoundManager *pSM, double dTimeCurrent )
{
	if(!m_initialized)
	{
		m_initialized = true;
		g_terrain.Create(pMA->GetDevice());
	

#ifdef UNIT_TESTING

		//Create unit test game objects
		GameObject* unittest1 = new GameObject( g_database.GetNewObjectID(), OBJECT_Ignore_Type, "UnitTest1" );
		GameObject* unittest2 = new GameObject( g_database.GetNewObjectID(), OBJECT_Ignore_Type, "UnitTest2" );
		GameObject* unittest3a = new GameObject( g_database.GetNewObjectID(), OBJECT_Ignore_Type, "UnitTest3a" );
		GameObject* unittest3b = new GameObject( g_database.GetNewObjectID(), OBJECT_Ignore_Type, "UnitTest3b" );
		GameObject* unittest4 = new GameObject( g_database.GetNewObjectID(), OBJECT_Ignore_Type, "UnitTest4" );
		
		D3DXVECTOR3 pos(0.0f, 0.0f, 0.0f);
		unittest1->CreateBody( 100, pos );
		unittest2->CreateBody( 100, pos );
		unittest3a->CreateBody( 100, pos );
		unittest3b->CreateBody( 100, pos );
		unittest4->CreateBody( 100, pos );
		
		g_database.Store( *unittest1 );
		g_database.Store( *unittest2 );
		g_database.Store( *unittest3a );
		g_database.Store( *unittest3b );
		g_database.Store( *unittest4 );

		//Give the unit test game objects a state machine
		unittest1->PushStateMachine( *new UnitTest1( *unittest1 ), true );
		unittest2->PushStateMachine( *new UnitTest2a( *unittest2 ), true );
		unittest3a->PushStateMachine( *new UnitTest3a( *unittest3a ), true );
		unittest3b->PushStateMachine( *new UnitTest3b( *unittest3b ), true );
		unittest4->PushStateMachine( *new UnitTest4( *unittest4 ), true );

#else
/*
	for( int i=0; i<10; i++ )
	{
		//Create game objects
		char name[10] = "NPC";
		sprintf( name, "%s%d", name, i );
		GameObject* npc = new GameObject( g_database.GetNewObjectID(), OBJECT_NPC, name );
		D3DXVECTOR3 pos(0.0f, 0.0f, 0.0f);
		pos.x = ((float)(rand()%100)) / 100.0f;
		pos.z = ((float)(rand()%100)) / 100.0f;
		npc->CreateBody( 100, pos );
		npc->CreateMovement();
		npc->CreateTiny( pMA, pv_pChars, pSM, dTimeCurrent );
		g_database.Store( *npc );

		//Give the game object a state machine
		npc->PushStateMachine( *new Example( *npc ), true );
	}
//*/

	{	//Agent
		GameObject* agent = new GameObject( g_database.GetNewObjectID(), OBJECT_Player, "Hero" );
		D3DXVECTOR3 pos(0.1f, 0.0f, 0.1f);
		agent->CreateBody( 100, pos );
		agent->CreateMovement();
		agent->CreateTiny( pMA, pv_pChars, pSM, dTimeCurrent );
		g_database.Store( *agent );
		agent->PushStateMachine( *new Agent( *agent ), true );
		g_terrain.m_player = agent;
	}
	{
		GameObject* commander = new GameObject( g_database.GetNewObjectID(), OBJECT_NPC, "Commander" );
		D3DXVECTOR3 pos(3.9f, 0.0f, 3.9f);
		commander->CreateBody( 100, pos );
		commander->CreateMovement();
		commander->CreateTiny( pMA, pv_pChars, pSM, dTimeCurrent );
		g_database.Store( *commander );
		commander->PushStateMachine( *new Commander( *commander ), true );
		//((Commander*)(commander->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(3.8f, 0, 3.8f));
		((Commander*)(commander->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(3.9f, 0, 3.9f));

		for(int b = 0; b < 2; ++b)
		{
			GameObject* leader = new GameObject( g_database.GetNewObjectID(), OBJECT_NPC, "Leader" );
			D3DXVECTOR3 pos(0.1f, 0, 3.1f);
			if(b == 1) pos = D3DXVECTOR3(1.0f, 0.0f, 1.0f);
			leader->CreateBody( 100, pos );
			leader->CreateMovement();
			leader->CreateTiny( pMA, pv_pChars, pSM, dTimeCurrent );
			g_database.Store( *leader );
			leader->PushStateMachine( *new Leader( *leader ), true );
			if(b == 0)
			{
				((Leader*)(leader->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(0.1f, 0, 3.8f));
				((Leader*)(leader->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(1.6f, 0, 3.2f));
				((Leader*)(leader->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(0.1f, 0, 3.1f));
			}
			else if(b == 1)
			{
				((Leader*)(leader->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(2.0f, 0, 1.0f));
				((Leader*)(leader->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(2.0f, 0, 0.4f));
				((Leader*)(leader->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(1.0f, 0, 0.4f));
				((Leader*)(leader->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(1.0f, 0, 1.0f));
			}
			((Leader*)(leader->GetStateMachine()))->RegisterSquadCommander(commander);
			((Leader*)(leader->GetStateMachine()))->RegisterIndex(b);
			((Commander*)(commander->GetStateMachine()))->RegisterSquad(leader);

			for(int c = 0; c < 2; ++c)
			{
				GameObject* agent = new GameObject( g_database.GetNewObjectID(), OBJECT_NPC, "Private" );
				D3DXVECTOR3 pos;
				if(b == 0)
				{
					if(c == 0) pos = D3DXVECTOR3(0.2f,0,3.2f);
					if(c == 1) pos = D3DXVECTOR3(0.0f,0,3.0f);
				}
				else if(b == 1) 
				{
					if(c == 0) pos = D3DXVECTOR3(0.8,0,0.8);
					if(c == 0) pos = D3DXVECTOR3(0.9,0,0.9);
				}
				agent->CreateBody( 100, pos );
				agent->CreateMovement();
				agent->CreateTiny( pMA, pv_pChars, pSM, dTimeCurrent );
				g_database.Store( *agent );
				agent->PushStateMachine( *new Private( *agent ), true );
				if(b == 0)
				{
					if(c == 0)
					{
						((Private*)(agent->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(0.2f, 0, 3.9f));
						((Private*)(agent->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(1.7f, 0, 3.3f));
						((Private*)(agent->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(0.2f, 0, 3.2f));
					}
					if(c == 1)
					{
						((Private*)(agent->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(0.0f, 0, 3.7f));
						((Private*)(agent->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(1.5f, 0, 3.1f));
						((Private*)(agent->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(0.0f, 0, 3.0f));
					}
				}
				else if(b == 1)
				{
					if(c == 0)
					{
						((Private*)(agent->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(1.8f, 0, 0.8f));
						((Private*)(agent->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(1.8f, 0, 0.2f));
						((Private*)(agent->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(0.8f, 0, 0.2f));
						((Private*)(agent->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(0.8f, 0, 0.8f));
					}
					if(c == 1)
					{
						((Private*)(agent->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(1.9f, 0, 0.9f));
						((Private*)(agent->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(1.9f, 0, 0.3f));
						((Private*)(agent->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(0.9f, 0, 0.3f));
						((Private*)(agent->GetStateMachine()))->RegisterPatrolSpot(D3DXVECTOR3(0.9f, 0, 0.9f));
					}
				}
				((Private*)(agent->GetStateMachine()))->RegisterSquadCommander(commander);
				((Private*)(agent->GetStateMachine()))->RegisterSquadLeader(leader);
				((Private*)(agent->GetStateMachine()))->RegisterIndex(b);	//IMPORTANT: Reigster SQUAD Index, not Individual Index!
				((Leader*)(leader->GetStateMachine()))->RegisterSquadMember(agent);
			}
		}
	}
#endif

	}
}


void World::PostInitialize()
{
	g_database.Initialize();
}

void World::OnLostDevice( void )
{
	g_debugdrawing.OnLostDevice();
}

void World::Update()
{
	g_time.MarkTimeThisTick();
	g_database.Update();
	g_msgroute.DeliverDelayedMessages();
}

void World::Animate( double dTimeDelta )
{
	g_database.Animate( dTimeDelta );
}

void World::AdvanceTimeAndDraw( IDirect3DDevice9* pd3dDevice, D3DXMATRIX* pViewProj, double dTimeDelta, D3DXVECTOR3 *pvEye )
{
	g_database.AdvanceTimeAndDraw( pd3dDevice, pViewProj, dTimeDelta, pvEye );
	g_terrain.DrawDebugVisualization( pd3dDevice );
}

void World::RestoreDeviceObjects( LPDIRECT3DDEVICE9 pd3dDevice )
{
	return( g_database.RestoreDeviceObjects( pd3dDevice ) );
}

void World::InvalidateDeviceObjects( void )
{
	g_database.InvalidateDeviceObjects();
}
void World::SetGoal(IDirect3DDevice9* pd3dDevice, D3DVIEWPORT9* viewport, D3DXMATRIX* mxView,
					D3DXMATRIX* mxProj, D3DXMATRIX* mxWorld, D3DXVECTOR3* vCamera, D3DXVECTOR3* vTarget)
{
	D3DXVECTOR3 mouse, wSpace, rayDir, endPt, intersection;
	D3DXPLANE tempPlane;
	D3DXMATRIX ident;
	D3DXMatrixIdentity(&ident);
	int i;
	mouse.x = (float)mouseCoords.x;
	mouse.y = (float)mouseCoords.y;
	mouse.z = 0;
	//this->m_terrain
	D3DXVec3Unproject(&wSpace, &mouse, viewport, mxProj, mxView, &ident);
	//D3DXVec3Unproject(&wSpace, &mouse, viewport, mxView, mxProj, mxWorld);

	rayDir = (wSpace - (*vCamera)) * 20.0f;
	//D3DXVec3Scale(&rayDir, &rayDir, 500.0f);
	//D3DXVec3Add(&endPt, &wSpace, &rayDir);
	D3DXVec3Add(&endPt, &wSpace, &rayDir);
	
	D3DXVECTOR3 origin, norm;
	origin.x=origin.y=origin.z=norm.x=norm.z=0;
	norm.y=1;
	D3DXPlaneFromPointNormal(&tempPlane, &origin, &norm);
	D3DXPlaneIntersectLine(&intersection, &tempPlane, vCamera, &endPt);
	if((&intersection) != NULL)
	{
		if( intersection.x >= 0.0f && intersection.x <= WORLD_SIZE && intersection.z >= 0.0f && intersection.z <= WORLD_SIZE )
		{
			int row, col;
			g_terrain.GetRowColumn(&intersection, &row, &col);
			dbCompositionList list, npcList;
			g_database.ComposeList(list, OBJECT_Player);
			g_database.ComposeList(npcList, OBJECT_NPC);
			//list[0]->GetMovement
			list[0]->GetMovement().goalr = row;
			list[0]->GetMovement().goalc = col;
			list[0]->GetMovement().m_usePickedGoal = true;
			//check against enemy places
			for(i=0; i < npcList.size(); ++i)
			{
				g_terrain.GetRowColumn(&(npcList[i]->GetBody().GetPos()), &row, &col);
				if(list[0]->GetMovement().goalr == row && list[0]->GetMovement().goalc == col)
				{
					D3DXVECTOR3 direction = g_terrain.m_player->GetBody().GetPos() - npcList[i]->GetBody().GetPos();
					//if(g_terrain.LOSCheck(&npcList[i]->GetBody().GetPos(), &direction) == NULL)
					{
						break;
					}
				}
			}
			if(i == npcList.size())
				g_database.SendMsgFromSystem(list[0]->GetID(), MSG_NextGoal);
			else
				g_database.SendMsgFromSystem(list[0]->GetID(), MSG_Kill);
		}
	}
}
